<?php

namespace HCM\FPA\ObrigacaoLegal\CNAB\Bradesco;

use HCM\Util\FPA;
use HCM\Util\Repositories;
use HCM\Util\DateUtil;
use HCM\Util\Util;
use HCM\FPA\Entity\FpaFormcnabEntity;
use HCM\FPA\Entity\FpaCnabmagnetEntity;
use HCM\FPA\ObrigacaoLegal\CNAB\Bradesco\Cnab200;

class Cnab200237 extends Cnab200{

    private $bufferSaida = '';      //Local da string de saída (arquivo)
    private $contador = 2;          //Contador de seqüência
    private $arquivo;

    public function setCaminhoRelativo($caminho){
        $this->caminhoRelativo = $caminho;
    }

    public function getCaminhoRelativo(){
        return $this->caminhoRelativo;
    }

    public function geraCabecalho(){
        
        if (isset($this->estruturas[0]['CDBANCO']) && $this->cdbanco == $this->estruturas[0]['CDBANCO']) {
            $this->preparaArquivo();

            $this->contador = 2;

            $campoCodReg = '0';
            $campoCodRem = '1';
            $campoLiter1 = 'REMESSA';
            $campoCodSer = '03';
            $campoLiter2 = $this->completaCaracter('CREDITO C/C', 15, ' ');
            $campoIdenti = $this->completaCaracter($this->estruturas[0]['CDAGENCIA'], 5, '0', false);
            $campoRazaoS = $this->razaoConta($this->estruturas[0]['NRTPCONTCORR']);
            $campoEmpres = $this->completaCaracter(substr($this->estruturas[0]['CDCONTCORR'], 0, -1), 7, '0', false); //Conta corrente
            $campoNoBanc = $this->completaCaracter(substr($this->estruturas[0]['CDCONTCORR'], -1), 1, '0'); //Dígito
            $campoID     = ' ';
            $campoReser1 = ' ';
            $campoCodEmp = $this->completaCaracter($this->codconvenio, 5, ' '); //Código da empresa fornecido pelo banco
            $campoNomEmp = $this->completaCaracter($this->estruturas[0]['NMRAZSOCESTRUT'], 25, ' ', true);
            $campoCodBan = $this->completaCaracter($this->cdbanco, 3, '0', false);
            $campoNomBan = $this->completaCaracter('BRADESCO', 15, ' ');
            $campoDtGrav = date('dmY');
            $campoDensid = '01600';
            $campoLiter3 = 'BPI';
            // Alteracao do Marcelo 18-08-2014 referente ao formato incorreto da data de pgto do cnab bradesco
            $auxCampoDtDebi     = DateUtil::getDataDeString(substr($this->dtpagamento, 0, 10));
            
            $campoDtDebi = $auxCampoDtDebi->format('dmY');			

            $campoIdMoed = ' ';
            $campoIdSecu = 'N';
            $campoReser2 = $this->completaCaracter(' ', 74, ' ');
            $campoNumSeq = '000001'; //Número sequencila. Tratar se for ter mais cabeçalhos.

            $this->bufferSaida = $campoCodReg.$campoCodRem.$campoLiter1.$campoCodSer.$campoLiter2.$campoIdenti.
                $campoRazaoS.$campoEmpres.$campoNoBanc.$campoID.$campoReser1.$campoCodEmp.$campoNomEmp.
                $campoCodBan.$campoNomBan.$campoDtGrav.$campoDensid.$campoLiter3.$campoDtDebi.$campoIdMoed.
                $campoIdSecu.$campoReser2.$campoNumSeq./*PHP_EOL*/"\r\n";
            $this->bufferSaida = $this->remove_utf8_bom($this->bufferSaida);
            
            FPA::escreveArquivo($this->nomeArquivo, $this->bufferSaida);
    
            //Verifica se deve gerar os registros no magnético
            if($this->idgerameiomag == 'S' && $this->idteste == 'N') {
                $this->gravaLinhaNoMagnetico();
            }
        } else {
            throw new \Exception('Estrutura não possui conta corrente para o banco informado.');
        }
    }

    public function geraRegistros(){
        $campoCodReg = '1';
        $campoReser1 = $this->completaCaracter(' ', 61, ' ');

        $campoReser2 = $this->completaCaracter(' ', 2, ' ');

        $campoCodSer = '298';
        $campoReser3 = $this->completaCaracter(' ', 8, ' ');
        $campoReser4 = $this->completaCaracter(' ', 44, ' ');

        foreach($this->vinculos as $vinculo){
            $campoAgenci = $this->completaCaracter($vinculo['CDAGENCIA'], 5, '0', false);
            $campoRazCon = $this->razaoConta($vinculo['NRTPCONTCORR']);
            $campoNumCon = $this->completaCaracter(substr($vinculo['CDCONTCORR'], 0, -1), 7, '0', false);
            $campoDigCon = substr($vinculo['CDCONTCORR'], -1);

            $campoNomFun = $this->completaCaracter($vinculo['NMVINCULOM'], 38, ' ');
            $campoCodFun = $this->completaCaracter($vinculo['CDMATRICULA'], 6, '0', false);
            $campoValor  = $this->completaCaracter(number_format($vinculo['VREVENTOFOLHA'], 2, '', ''), 13, '0', false);

            $campoNumSeq = $this->completaCaracter(number_format($this->contador++, 0, '', ''), 6, '0', false);

            $this->bufferSaida = $campoCodReg.$campoReser1.$campoAgenci.$campoRazCon.$campoNumCon.$campoDigCon.
                $campoReser2.$campoNomFun.$campoCodFun.$campoValor.$campoCodSer.$campoReser3.$campoReser4.
                $campoNumSeq./*PHP_EOL*/"\r\n";
            
            FPA::escreveArquivo($this->nomeArquivo, $this->bufferSaida);

            //Verifica se deve gerar os registros no magnético
            if($this->idgerameiomag == 'S' && $this->idteste == 'N') {
                $this->gravaLinhaNoMagnetico($vinculo['VREVENTOFOLHA'], $vinculo['NRVINCULOM']);
            }
        }
    }

    /*
     * Gera o rodapé do CNAB. Fecha o arquivo. Inicia o download do mesmo
     */
    public function geraRodape(){
        $campoCodReg = '9';
        $campoTotVal = $this->completaCaracter(number_format($this->vrtotal, 2, '', ''), 13, '0', false);
        $campoReserv = $this->completaCaracter(' ', 180, ' ');
        $campoNumSeq = $this->completaCaracter(number_format($this->contador, 0, '', ''), 6, '0', false);

        $this->bufferSaida = $campoCodReg.$campoTotVal.$campoReserv.$campoNumSeq;
        
        FPA::escreveArquivo($this->nomeArquivo, $this->bufferSaida);    
        
        //Verifica se deve gerar os registros no magnético
        if($this->idgerameiomag == 'S' && $this->idteste == 'N') {
            $this->gravaLinhaNoMagnetico();
        }
        
        return array(
            'ARQUIVO' => $this->nomeArquivo,
            'CALCULO' => null,
            'MENSAGENS' => null,
            'ERRO' => null,
            'ABORT' => null
        );
    }

    /*
     * De - Para para o tipo de conta
     */
    protected function razaoConta($tipoConta){
        switch($tipoConta){
            case '3':
            case '1':
                return '07050';
            case '2':
            //case '3':
                return '07380';
            default:
                return '00000';
        }
    }

    /*
     * Cria a estrutura de pastas do caminho relativo se necessário.
     * Exclui todos os arquivos da pasta relativa.
     * Gera o nome do arquivo com o caminho relativo para o novo cnab considerando a data e o número sequencial.
     *
     */
    private function preparaArquivo(){

        //Inicio da montagem do nome do arquivo
        $this->nomeArquivo = 'FP'.date('dm');

        $form = new FpaFormcnabEntity($this->entityManager, $this->hcmParameters);
        $dadosFormulario = $form->retornaDadosFormularioCNAB($this->nrorg, $this->nrformcnab);
        $data = $dadosFormulario->getDtultimocnab();
        $nro  = $dadosFormulario->getNrultimocnab();

        //Número sequencial do arquivo, de 0 a 9
        if($data == date('d/m/Y 00:00:00'))
        {
            ++$nro;
            if($nro > 9)
                $nro = 0;
        } else {
            $data = date('dmY');
            $nro = 0;
        }
        $this->nomeArquivo .= $nro;
        $this->nrSeqArquvio = $nro;

        //Extensão do arquivo
        if($this->idteste == 'S')
            $this->nomeArquivo .= '.TST';
        else
            $this->nomeArquivo .= '.TXT';

        //Grava a data e o número sequencial do cnab gerado no formulário
        // $dadosFormulario->setDtultimocnab($data);
        $dadosFormulario->setDtultimocnab(DateUtil::getDataDeString(date('dmY'), DateUtil::FORMATO_BRASILEIRO, true));
        $dadosFormulario->setNrultimocnab($nro);
        $this->entityManager->persist($dadosFormulario);
        $this->entityManager->flush($dadosFormulario);
    }

    public function getNomeArquivoGerado(){
        return $this->nomeArquivo;
    }

    public function gravaLinhaNoMagnetico($valor = null, $nrvinculom = null){
        $magnetico = new FpaCnabmagnetEntity($this->entityManager, $this->hcmParameters);

        //verificar se já tem uma data igual a esta e retroceder se tiver
        //Armazenar o CNAB
        //$teste = new \FpaCnab

        $magnetico->insereRegistroLista($this->nrorg, $this->cdOperador, $this->competencia, null, $this->idtpinscrespons,
            $this->estruturas[0]['CDCNPJESTRUT'], $this->bufferSaida, $this->cdbanco, $this->nrtipomoviment,
            $this->nrSeqArquvio, $valor, $nrvinculom);

        $magnetico->persist();
    }
}