<?php

namespace HCM\Geral\Logic;

use HCM\Geral\Logic\Esocial\ESocialDataFactory;
use HCM\Geral\Logic\Esocial\ESocialTransmissor;
use HCM\Geral\Logic\Esocial\ESocialXmlFactory;
use HCM\Util\DateUtil;
use HCM\Util\FileUploader;
use HCM\Util\Messages;
use HCM\Util\MsgProcesso;
use HCM\Util\Repositories;

/**
 * Created by PhpStorm.
 * User: felipesantos
 * Date: 11/10/2017
 * Time: 10:00
 */

class ESocialLogic {

    /* Grupos de Eventos do eSocial */
    const IDGRUPO_CADASTRO = 'C';
    const IDGRUPO_INICIAL = 'I';
    const IDGRUPO_TABELAS = 'T';
    const IDGRUPO_NAOPERIODICO = 'N';
    const IDGRUPO_PERIODICO = 'P';

    /* Operações Tela Envios */
    const IDTRANSENVIO = 'ENVIAR'; //Enviar Evento
    const IDTRANSCONSU = 'CONSULTAR'; //Consultar Status do evento
    const IDTRANSREENV = 'REENVIAR'; //Reenviar Evento
    const IDTRANSEXCLU = 'EXCLUSAO'; //Excluir Evento
    /* Operações Tela BX */
    const IDCONSUIDEMPR = 'CONSULTAIDEMPR'; //Consulta Identificadores Empregador
    const IDCONSUIDTABE = 'CONSULTAIDTABE'; //Consulta Identificadores Empregador
    const IDCONSUIDTRAB = 'CONSULTAIDTRAB'; //Consulta Identificadores Empregador
    /* Operações Download Eventos */
    const IDDOWNLOADID = 'DOWNLOADID'; //Download de Evento por ID
    const IDDOWNLOADREC = 'DOWNLOADREC'; //Download de Evento por Recibo
    
    /* Status dos eventos */
    const IDSTATUS_PENDENTE = 'P';
    const IDSTATUS_ERRO_ENVIO = 'E';
    const IDSTATUS_PROCESSAMENTO = 'M';
    const IDSTATUS_ERRO_PROCESSAMENTO = 'R';
    const IDSTATUS_CONCLUIDO = 'C';
    const IDSTATUS_CONCLUIDO_ADVERTENCIA = 'A';
    
    /* Códigos de retorno do envio do lote */
    const CD_RET_LOTE_201 = '201';     // Lote recebido com sucesso
    const CD_RET_LOTE_202 = '202';     // Lote recebido com advertências
    const CD_RET_LOTE_301 = '301';     // Erro servidor eSocial
    const CD_RET_LOTE_401 = '401';     // Lote incorreto - Erro preenchimento
    const CD_RET_LOTE_402 = '402';     // Lote incorreto - Schema inválido
    const CD_RET_LOTE_403 = '403';     // Lote incorreto - Versão do schema não permitida
    const CD_RET_LOTE_404 = '404';     // Lote incorreto - Erro certificado
    const CD_RET_LOTE_405 = '405';     // Lote incorreto - Lote nulo ou vazio
    /* Códigos de retorno da consulta de processamento do lote */
    const CD_RET_CONS_101 = '101';     // Lote aguardando processamento
    const CD_RET_CONS_201 = '201';     // Lote Processado com Sucesso.
    const CD_RET_CONS_202 = '202';     // Lote Processado com Advertências.
    const CD_RET_CONS_301 = '301';     // Erro Servidor eSocial
    const CD_RET_CONS_401 = '401';     // Lote Incorreto - Erro preenchimento.
    const CD_RET_CONS_402 = '402';     // Lote Incorreto - Schema Inválido.
    const CD_RET_CONS_403 = '403';     // Lote Incorreto - Versão do Schema não permitida.
    const CD_RET_CONS_404 = '404';     // Lote Incorreto - Erro Certificado.
    const CD_RET_CONS_405 = '405';     // Lote Incorreto - Lote nulo ou vazio.
    const CD_RET_CONS_501 = '501';     // Solicitação de Consulta Incorreta -Erro Preenchimento.
    const CD_RET_CONS_502 = '502';     // Solicitação de Consulta Incorreta - Schema Inválido.
    const CD_RET_CONS_503 = '503';     // Solicitação de Consulta Incorreta - Versão do Schema Não Permitida.
    const CD_RET_CONS_504 = '504';     // Solicitação de Consulta Incorreta - Erro Certificado.
    const CD_RET_CONS_505 = '505';     // Solicitação de Consulta Incorreta - Consulta nula ou vazia.
    /* Códigos de retorno de processamento do evento */
    const CD_RET_EVEN_201 = '201';     // Sucesso
    const CD_RET_EVEN_202 = '202';     // Sucesso com advertência
    const CD_RET_EVEN_301 = '301';     // Erro Servidor
    const CD_RET_EVEN_401 = '401';     // Erro no conteúdo do evento
    const CD_RET_EVEN_402 = '402';     // Schema inválido
    const CD_RET_EVEN_403 = '403';     // Leiaute inválido
    const CD_RET_EVEN_404 = '404';     // Erro do certificado digital da assinatura do evento
    const CD_RET_EVEN_405 = '405';     // Erro na assinatura evento
    const CD_RET_EVEN_406 = '406';     // Evento não pertence ao grupo especificado no lote de eventos
    const CD_RET_EVEN_411 = '411';     // Assinante inválido

    const ERRO = 'Erro';
    const ADVERTENCIA = 'Advertência';

    /** @var \Doctrine\ORM\EntityManager */
    private $entityManager;
    /** @var \HCM\Util\HCMParameters */
    private $hcmParameters;

    private $nrestruturam;
    
    private $nrorg;
    private $dtmescompetenc;
    
    private $nrEstruEmpregador;
    private $nrParcEmpregador;
    private $tpInscEmpregador;
    private $nrInscEmpregador;
    private $cdNatuJuri;
    
    private $dsVersao;
    private $tpAmbiente;
    private $dsAmbiente;
    
    private $nrParcTransmissor;
    private $tpInscTransmissor;
    private $nrInscTransmissor;
    
    private $certFile;
    private $certPass;
    private $certKeyFile;
    private $publicKey;
    private $privateKey;
    
    private $idGrupo;
    
    private $entityEnvio;
    
    private $idOperacao;
    
    private $messages;
    
    private $tagNivel;
    private $tagsTrad;
    private $tagsOpc;

    /** @var ESocialLoteLogic */
    private $lote;

    public function __construct($entityManager, $hcmParameters, $nrestruturam, $nrtpambiente) {
        $this->entityManager = $entityManager;
        $this->hcmParameters = $hcmParameters;
        $this->nrestruturam = $nrestruturam;
            
        $this->tpAmbiente = $nrtpambiente;
        $this->dsAmbiente = $this->tpAmbiente == 1 ? 'Producao' : 'ProducaoRestrita';
        
        $this->nrorg = $this->hcmParameters->getNrorgTrabalho();
        $this->dtmescompetenc = $this->hcmParameters->getDtCompetencia();
        
        $this->tempEsocialPath = $this->hcmParameters->getTempDirectory() . DIRECTORY_SEPARATOR . "esocial";

        $this->certificatesPath = $this->hcmParameters->getFileServerDirectory();
        $this->tempCertificatesPath = $this->tempEsocialPath . DIRECTORY_SEPARATOR . "certificates" . DIRECTORY_SEPARATOR . $this->hcmParameters->getNrorgTrabalho();

        $this->xmlFilesPath = $this->tempEsocialPath . DIRECTORY_SEPARATOR . "xmlFiles" . DIRECTORY_SEPARATOR . $this->hcmParameters->getNrorgTrabalho();
        
        $this->setDadosEmpregador();

        $this->setDadosTransmissao();
        
        $this->messages = array();
            
        $this->tagsTrad = array();
        $this->tagsOpc = array();
    }
    
    private function setDadosEmpregador() {
        try {
            $estrutura = $this->entityManager->getRepository(Repositories::ESTRUTURAM)->getEstrutura(
                $this->hcmParameters->getNrorgTrabalho(), $this->nrestruturam, $this->hcmParameters->getDtcompetencia()
            );
            
            if ($estrutura) {
                $this->nrEstruEmpregador = $estrutura->getNrestruturam();
                if ($estrutura->historico->getCdcnpjestrut()) {
                    if (strlen($estrutura->historico->getCdcnpjestrut()) == 14) {
                        $this->tpInscEmpregador = '1';
                    } else if (strlen($estrutura->historico->getCdcnpjestrut()) == 11) {
                        $this->tpInscEmpregador = '2';
                    } else {
                        $this->tpInscEmpregador = '';
                    }
                    $this->nrInscEmpregador = $estrutura->historico->getCdcnpjestrut();
                } else {
                    $parcnegocio = $this->entityManager->getRepository(Repositories::PARCNEGOCIO)->findOneBy(
                        array('nrorg'=>$this->hcmParameters->getNrorgTrabalho(), 'nrparcnegocio'=>$estrutura->getNrparcnegocio())
                    );
                    $this->nrParcEmpregador = $parcnegocio->getNrparcnegocio();
                    if ($parcnegocio->getCdtipoinscricao() == 'CNPJ') {
                        $this->tpInscEmpregador = '1';
                    } else if ($parcnegocio->getCdtipoinscricao() == 'CPF') {
                        $this->tpInscEmpregador = '2';
                    } else {
                        $this->tpInscEmpregador = '';
                    }
                    $this->nrInscEmpregador = $parcnegocio->getNrinscricaoparc();
                }
                $this->cdNatuJuri = $estrutura->historico->getCdnatujuri();
            } else {
                throw new \Exception("Não foi encontrada a estrutura " . $this->nrestruturam);
            }
        } catch(\Exception $e) {
            throw $e;
        }
    }
    
    private function setDadosTransmissao() {
        try {
            $esoParametro = $this->entityManager->getRepository(Repositories::ESO_PARAMETRO)->findOneBy(
                array('nrorg'=>$this->hcmParameters->getNrorgTrabalho(), 'nrestruturam'=>$this->nrestruturam)
            );
                
            if (!is_object($esoParametro)) {
                throw new \Exception("Não foi encontrado parâmetro do eSocial para a estrutura " . $this->nrestruturam);
            }
            
            $this->dsVersao = $esoParametro->getDsversao();
            
            if (!$esoParametro->getNrparcnegocio()) {
                throw new \Exception("Responsável pela transmissão não definido nos parâmetros do eSocial da estrutura " . $this->nrestruturam);
            }
            
            $parcnegocio = $this->entityManager->getRepository(Repositories::PARCNEGOCIO)->findOneBy(
                array('nrorg'=>$this->hcmParameters->getNrorgTrabalho(), 'nrparcnegocio'=>$esoParametro->getNrparcnegocio())
            );
                    
            $this->nrParcTransmissor = $parcnegocio->getNrparcnegocio();
            if ($parcnegocio->getCdtipoinscricao() == 'CNPJ') {
                $this->tpInscTransmissor = '1';
            } else if ($parcnegocio->getCdtipoinscricao() == 'CPF') {
                $this->tpInscTransmissor = '2';
            } else {
                $this->tpInscTransmissor = '';
            }
            
            $this->nrInscTransmissor = $parcnegocio->getNrinscricaoparc();
            
            if (!$esoParametro->getDsarqucert()) {
                throw new \Exception(Messages::HCM09000_ESO_CERTIFICADO_NOT_FOUND_ERROR, Messages::ERROR_MESSAGE_CODE);
            }
            if (!$esoParametro->getDssenhacert()) {
                throw new \Exception(Messages::HCM09000_ESO_SENHA_CERTIFICADO_NOT_FOUND_ERROR, Messages::ERROR_MESSAGE_CODE);
            }

            $this->certFile = $this->certificatesPath . $esoParametro->getDsarqucert();
            $this->certKeyFile = $this->tempCertificatesPath . DIRECTORY_SEPARATOR . "certKey" . $this->nrInscTransmissor . ".pem";
            $this->certPass = $esoParametro->getDssenhacert();
            
            // SOLUÇÃO NOVO FILESERVER	 
            // $this->certFile = Util::buscaArquivoFileServer($this->certFile, '.pfx');	 
            // // $this->certFile é o conteúdo do arquivo buscado	 
            // // comentar linha "$this->certFile = file_get_contents($this->certFile);", pois já retornado o conteúdo do arquivo
            
            // Variável criada para realizar 5 tentativas de acessar o cetificado no FileServer antes de retornar o erro
            $openCertFile = false;
            
            /* Retira possíveis warnings do php ao usar fopen no fileServer */
            error_reporting(E_ALL ^ E_WARNING);
            for ($i=0;$i<5;$i++) {
                if (fopen($this->certFile,"r") == true) {
                    $openCertFile = true;
                    break;
                }
            }
            error_reporting(E_ALL ^ E_NOTICE);
            
            if ($openCertFile) {
                if (file_exists($this->certKeyFile)) {
                    unlink($this->certKeyFile);
                }
                if (!file_exists($this->certKeyFile)) {
                    if (!is_dir($this->tempCertificatesPath)) {
                        mkdir($this->tempCertificatesPath, 0777, true);
                    }
                    // Alteração temporária para ler somente uma vez o arquivo do FileServer e armazenar na variável certFile
                    //$data = file_get_contents($this->certFile);
                    //openssl_pkcs12_read($data, $cert, $this->certPass);
                    $this->certFile = file_get_contents($this->certFile);
                    openssl_pkcs12_read($this->certFile, $cert, $this->certPass);
                    
                    if (is_null($cert)) {
                        throw new \Exception(Messages::HCM09000_ESO_GENERATE_CERT_KEY_ERROR, Messages::ERROR_MESSAGE_CODE);
                    } else {
                        //Dados do certificado
                        $dados = array ();
                        $dados = openssl_x509_parse( openssl_x509_read($cert['cert']) );
                        
                        //Dados mais importantes
                        //$dados['subject']['C']; //País
                        //$dados['subject']['ST']; //Estado
                        //$dados['subject']['L']; //Município
                        //$dados['subject']['CN']; //Razão Social e CNPJ / CPF
                        $validadeCert = date('d/m/Y', $dados['validTo_time_t']);//Validade	
                        $validadeCert = DateUtil::getDataDeString($validadeCert, DateUtil::FORMATO_BRASILEIRO, true);
                        //$dados['extensions']['subjectAltName'];	//Emails Cadastrados separado por ,
                        
                        $dataAtual = date('d/m/Y');
                        $dataAtual = DateUtil::getDataDeString($dataAtual, DateUtil::FORMATO_BRASILEIRO, true);
                        // Verifica se a validade é anterior à data atual
                        if (DateUtil::comparaDatas($validadeCert, $dataAtual)) {
                            throw new \Exception(Messages::HCM09000_ESO_CERTIFICADO_VALIDADE_ERROR, Messages::ERROR_MESSAGE_CODE);
                        } else {
                            $this->publicKey = $this->getPublicKey($cert["cert"]);
                            $this->privateKey = openssl_get_privatekey($cert["pkey"]);
                            
                            file_put_contents($this->certKeyFile, $cert["pkey"] . "\r\n" . $cert["cert"]);
                            chmod($this->certKeyFile, 0777);
                        }
                    }
                }
            } else {
                throw new \Exception(Messages::HCM09000_ESO_CERTIFICADO_NOT_OPEN_ERROR, Messages::ERROR_MESSAGE_CODE);
            }
        } catch(\Exception $e) {
            throw $e;
        }
    }
    
    private function addMessage($message) {
        $this->messages[] = $message;
    }
    
    public function getMessages() {
        return $this->messages;
    }
    
    private function setTagsTraducao() {
        $tagsTrad = $this->entityManager->getRepository(Repositories::ESO_TAGNAME)->findBy(array('nrorg' => $this->hcmParameters->getNrorgPadrao()));
        
        foreach ($tagsTrad as $tag) {
            $this->tagsTrad[$tag->getDstagname()] = $tag->getDstagtraducao() != '' ?  $tag->getDstagtraducao() : $tag->getDstagname();
            $cdOpcoes = $tag->getDscdopctag() ? explode(';', $tag->getDscdopctag()) : null;
            $nmOpcoes = $tag->getDsnmopctag() ? explode(';', $tag->getDsnmopctag()) : null;
            if($cdOpcoes){
                foreach($cdOpcoes as $index => $cdOpcao){
                    $this->tagsOpc[$tag->getDstagname()][$cdOpcao] = $nmOpcoes[$index];
                }
            }
        }
    }

    private function getPublicKey($key) {
        $data = "";
        $arCert = explode("\n", $key);
        foreach ($arCert as $curData) {
            //remove a tag de inicio e fim do certificado
            if (strncmp($curData, "-----BEGIN CERTIFICATE", 22) != 0 &&
                strncmp($curData, "-----END CERTIFICATE", 20) != 0 ) {
                //carrega o resultado numa string
                $data .= trim($curData);
            }
        }
        return $data;
    }
    
    private function getEsoEvento($nmevento, $dsversao) {
        /** @var \Teknisa\Model\Gestaopessoas\Entities\EsoEvento $esoevento */
        $esoevento = $this->entityManager->getRepository(Repositories::ESO_EVENTO)->findOneBy(array(
            'nmevento' => $nmevento, 'dsversao' => $dsversao
        ));
        
        if (!is_object($esoevento)) {
            throw new \Exception('Registro de evento não encontrado. Favor atualizar os dados da tela e tentar novamente.');
        }
        
        return $esoevento;
    }
    
    /**
     * Gera ID para o evento
     * - Tamanho deve ser igual a 36 caracteres
     * - Deve ser iniciado por 'ID'
     * - O terceiro caracter deve ser 1 (CNPJ) ou 2 (CPF)
     * - CNPJ do empregador. Raíz do CNPJ ou CNPJ completo dependendo da Natureza Jurídica
     * - Data Hora da geração
     * - Sequencial de 5 dígitos com o final do NRENVIO
     * 
     * - Verifica se já existe outro envio com o mesmo ID gerado
     */
    private function geraIdEvento($nrestruturam, $nrenvio) {
        try {
            //Busca o empregador para pegar o CNPJ e a Natureza Jurídica
            $estruturah = $this->entityManager->getRepository(Repositories::ESTRUTURAH)->findOneBy(
                array('nrestruturam' => $nrestruturam, 'nrorg' => $this->hcmParameters->getNrorgTrabalho()),
                array('dtmescompetenc' => 'desc')
            );
            //CNPJ do empregador sem caracteres especiais
            $cdcnpjestrut = str_replace(array('.','/','-'), '', $estruturah->getCdcnpjestrut());
            
            $tipo = strlen($cdcnpjestrut) == 14 ? '1' : '2';
            if ($tipo == '1') {
                $inscricao = array_search($estruturah->getCdnatujuri(), array('1015','1040','1074','1163')) === false ? str_pad(substr($cdcnpjestrut, 0, 8), 14, '0', STR_PAD_RIGHT) : $cdcnpjestrut;
            } else {
                $inscricao = str_pad($cdcnpjestrut, 14, '0', STR_PAD_RIGHT);
            }
            $data = DateUtil::getDataAtual()->format('YmdHis');
            $seq = str_pad(substr($nrenvio, -5), 5, '0', STR_PAD_LEFT);
            
            //Monta o ID do evento: 'ID' + Tipo de inscrição + Inscrição + Data/Hora + Sequencial
            $idEvento = 'ID' . $tipo . $inscricao . $data . $seq;
            
            //Valida a formação do ID
            if (strlen($idEvento) != 36 || substr($idEvento,0,2) != 'ID' || (substr($idEvento,2,1) != '1' && substr($idEvento,2,1) != '2') || 
                (strlen(str_replace(array('.','/','-'), '', substr($idEvento,3,14))) != 11 && strlen(str_replace(array('.','/','-'), '', substr($idEvento,3,14))) != 14)) {
                throw new \Exception(Messages::HCM09000_ESO_IDEVENTO_INVALID_ERROR,Messages::ERROR_MESSAGE_CODE);
            }
            
            // Verifica se já existe outro envio com o mesmo ID
            $envios = $this->entityEnvio->findBy(array('idevento' => $idEvento, 'nrorg' => $this->hcmParameters->getNrorgTrabalho()));
            if (count($envios) > 0) {
                throw new \Exception(Messages::HCM09000_ESO_IDEVENTO_INVALID_ERROR,Messages::ERROR_MESSAGE_CODE);
            }
            return $idEvento;
        } catch (\Exception $e) {
            throw $e;
        }
    }
    
    private function getXmlEvento($dtsName, $dsVersao, $dados, $geraIdEvento = false, $geraVarAmb = false, $exclusaoEmpregador = false) {
        $tpInsc = $this->tpInscEmpregador;
        if (array_search($this->cdNatuJuri, array('1015','1040','1074','1163')) !== false && $this->tpInscEmpregador == '1') {
            $nrInsc = $this->nrInscEmpregador;
        } else {
            $nrInsc = substr($this->nrInscEmpregador,0,8);
        }
        
        /* Monta o array de dados para o xml */
        $dataFactory = new ESocialDataFactory($this->entityManager, $this->hcmParameters, $dtsName, $dsVersao, $tpInsc, $nrInsc);
        $dadosXml = $dataFactory->buildDadosXml($dados, null, $geraIdEvento, $geraVarAmb, null);
        if (!$dadosXml) {
            throw new \Exception("Não foram encontrados dados para envio. Favor verificar o cadastro de origem dos dados",Messages::ERROR_MESSAGE_CODE);
        }
        
        /* Seta os dados específicos para identificar a operação de exclusão dos dados do empregador */
        if ($exclusaoEmpregador) {
            $dadosXml['evtInfoEmpregador']['infoEmpregador']['inclusao']['infoCadastro']['CLASSTRIB'] = '00';
        }
        
        /* Monta o xml do evento*/
        $xmlFactory = new ESocialXmlFactory($dtsName, $dsVersao);
        $xmlEvento = $xmlFactory->buildXml($dadosXml);
        
        /* Assina o xml do evento */
        $xmlEvento = $xmlFactory->assinaXmlEvento($xmlEvento, $this->publicKey, $this->privateKey);
        
        //$retorno = $xmlFactory->validaSchemaXml($xmlEvento);
        
        return $xmlEvento;
    }
    
    private function getXmlConsulta($nrProtocolo, $dsVersao) {
        try {
            $xmlData = array(
                "NRPROTOCOLO" => $nrProtocolo,
            );
            
            $dtsName = "consultaLoteEventos";
            
            /* Monta o xml de consulta */
            $xmlFactory = new ESocialXmlFactory($dtsName, $dsVersao);
            $xmlConsulta = $xmlFactory->buildXml($xmlData);
            
            return $xmlConsulta;
        } catch(\Exception $e) {
            throw $e;
        }
    }
    
    private function getXmlLote($idGrupo, $dsVersao, $eventosLote) {
        try {
            $tpInsc = $this->tpInscEmpregador;
            if (array_search($this->cdNatuJuri, array('1015','1040','1074','1163')) !== false && $this->tpInscEmpregador == '1') {
                $nrInsc = $this->nrInscEmpregador;
            } else {
                $nrInsc = substr($this->nrInscEmpregador,0,8);
            }
        
            $xmlData = array(
                "IDGRUPO" => ($idGrupo == 'P' ? '3' : ($idGrupo == 'N' ? '2' : '1')),
                "EMPTPINSCRICAO" => $tpInsc,
                "EMPNRINSCRICAO" => $nrInsc,
                "TRANSMTPINSCRICAO" => $this->tpInscTransmissor,
                "TRANSMNRINSCRICAO" => $this->nrInscTransmissor,
                "EVENTOS" => $eventosLote,
            );
            
            $xmlFactory = new ESocialXmlFactory("envioLoteEventos", $dsVersao);
            $xmlLote = $xmlFactory->buildXml($xmlData);
            
            return $xmlLote;
        } catch(\Exception $e) {
            throw $e;
        }
    }

    /**
     * @param $nrparcnegocio
     * @param $nranexo
     * @param $fileContent
     * @param $fileName
     * @param $dsreferencia
     * @return string
     * @throws \Exception
     */
    private function saveXmlOnFileServer($fileContent, $fileName, $dsreferencia, $custom=null) {
        if ($this->idGrupo == 'I' || $this->idGrupo == 'T') {
            $dsGrupo = 'Cadastros';
        } else if ($this->idGrupo == 'N') {
            $dsGrupo = 'NaoPeriodicos';
        } else if ($this->idGrupo == 'P') {
            $dsGrupo = 'Periodicos';
        } else {
            $dsGrupo = 'Outros';
        }
        
        $filePath = $this->dsAmbiente . "/" . $dsGrupo . "/" . $this->nrInscEmpregador . "/" . $this->idOperacao;
        
        if (!is_null($custom)) {
            $filePath = $filePath . "/" . $custom;
        }
        
        $row = array(
            "DSPATH" => array(
                array(
                    "name"               => $fileName.".xml",
                    "lastModified"       => 1507122512763,
                    "lastModifiedDate"   => DateUtil::getDataAtual()->format(DateUtil::FORMATO_AMERICANO_DATAHORA),
                    "webkitRelativePath" => "",
                    "size"               => strlen($fileContent),
                    "type"               => "application/xml",
                    "b64File"            => "data:application/xml;base64," . (string)base64_encode($fileContent),
                ),
            ),
            "NRORG"          => $this->hcmParameters->getNrorgTrabalho(),
            "NRPARAMETRO"    => "",
            "NRPARCNEGOCIO"  => $this->nrParcEmpregador,
            "NRANEXO"        => null,
            "CDOPERINCLUSAO" => $this->hcmParameters->getCdoperador(),
            "CDOPERADOR_LOG" => $this->hcmParameters->getCdoperador(),
            "DSREFERENCIA"   => $dsreferencia,
            "IDFOTO"         => 'N',
        );
        
        $fileFieldName = "DSPATH";
        
        FileUploader::uploadFilesInRow($row, $fileFieldName, $filePath);
        //$upload = $this->uploadsFileService->setGpeAnexos($row); @TODO Save on GPE_ANEXOS
        
        $filePath = $row['DSPATH'][0]['path'];
        
        return $filePath;
    }
    
    private function consultarIdentificadoresEventos($bxEsocial, $dtsName, $dtsLote, $dsVersao, $dados) {
        /* Monta e assina o xml do evento */
        $xmlEvento = $this->getXmlEvento($dtsName, $dsVersao, $dados);
        
        /* Salva o xml no FileServer */
        $fileName = $bxEsocial->getNrbxesocial() . DateUtil::getDataAtual()->format('YmdHis');
        $xmlPath = $this->saveXmlOnFileServer($xmlEvento, $fileName, $fileName);

        $bxEsocial->setDtenvio(DateUtil::getDataAtual());
        $bxEsocial->setDsdirxmlenv($xmlPath);
        $this->entityManager->persist($bxEsocial);
        
        /* Monta o xml do lote de consulta */
        $xmlFactory = new ESocialXmlFactory($dtsLote, $dsVersao);
        $xmlLote = $xmlFactory->buildXml(array('XMLEVENTO' => $xmlEvento));
        
        /* Realiza a transmissão do xml para o webservice do esocial*/
        $transmissor = new ESocialTransmissor($this->tpAmbiente, $this->idOperacao, $this->certKeyFile);
        $xmlResult = $transmissor->transmitirXml($xmlLote);
        
        /* Trata o retorno da consulta */
        $retorno = $this->trataRetornoConsultaIdentificadoresEventos($bxEsocial, $xmlResult);
        
        return $retorno;
    }
    
    private function trataRetornoConsultaIdentificadoresEventos($bxEsocial, $xmlResult) {
        try {
            $codRetorno = null;
            $idStatus = null;
            $cdErro = null;
            $msg = null;
            
            $xmlObj = simplexml_load_string($xmlResult);

            $codRetorno = (string) $xmlObj->retornoConsultaIdentificadoresEvts->status->cdResposta;
            $msg = (string) $xmlObj->retornoConsultaIdentificadoresEvts->status->descResposta;
            switch($codRetorno) {
                case '201':
                    $idStatus = self::IDSTATUS_CONCLUIDO;
                    $cdErro = null;
                    $this->trataRetornoIdentificadoresEvts($bxEsocial, $xmlObj->retornoConsultaIdentificadoresEvts->retornoIdentificadoresEvts);
                    break;
                case '203':
                    if ($xmlObj->retornoConsultaIdentificadoresEvts->retornoIdentificadoresEvts->qtdeTotEvtsConsulta > 50) {
                        $idStatus = self::IDSTATUS_PROCESSAMENTO;
                    } else {
                        $idStatus = self::IDSTATUS_CONCLUIDO;
                    }
                    $cdErro = null;
                    $this->trataRetornoIdentificadoresEvts($bxEsocial, $xmlObj->retornoConsultaIdentificadoresEvts->retornoIdentificadoresEvts);
                    break;
                case '301':
                case '307':
                case '308':
                case '309':
                case '310':
                    $idStatus = self::IDSTATUS_PENDENTE;
                    $cdErro = (string) $xmlObj->retornoConsultaIdentificadoresEvts->status->cdResposta;
                    break;
                case '402':
                case '403':
                case '404':
                case '405':
                case '406':
                case '407':
                case '409':
                case '410':
                case '411':
                case '417':
                default:
                    $idStatus = self::IDSTATUS_PENDENTE;
                    $cdErro = (string) $xmlObj->retornoConsultaIdentificadoresEvts->status->cdResposta;
                    break;
            }
            
            /* Salva o xml no FileServer */
            $fileName = $bxEsocial->getNrbxesocial() . 'Retorno' . DateUtil::getDataAtual()->format('YmdHis');
            $xmlPath = $this->saveXmlOnFileServer($xmlResult, $fileName, $fileName);
            
            $bxEsocial->setDtresposta(DateUtil::getDataAtual());
            $bxEsocial->setDsdirxmlretenv($xmlPath);
            $bxEsocial->setIdstatus($idStatus);
            $bxEsocial->setCderro($cdErro);
            $bxEsocial->setDsobservacao(substr($msg,0,4000));
            $bxEsocial->setDtultatu(DateUtil::getDataAtual());
            $bxEsocial->setCdoperultatu($this->hcmParameters->getCdoperador());
            $bxEsocial->setNrorgultatu($this->hcmParameters->getNrorgTrabalho());
            $this->entityManager->persist($bxEsocial);
            $this->entityManager->flush();

            $this->addMessage($codRetorno . '-' . $msg);
            
            if (is_null($cdErro)) {
                return true;
            } else {
                return false;
            }
        } catch(\Exception $e) {
            throw $e;
        }
    }
    
    private function trataRetornoIdentificadoresEvts($bxEsocial, $xmlObj) {
        try {
            $qtdTotEvts = 0;
            $dhUltimoEvt = null;
            
            $qtdTotEvts = $xmlObj->qtdeTotEvtsConsulta;
            if ($xmlObj->dhUltimoEvtRetornado) {
                $dhUltimoEvt = str_replace('T', ' ', substr((string)$xmlObj->dhUltimoEvtRetornado, 0, 19));
                $dhUltimoEvt = DateUtil::getDataDeString($dhUltimoEvt, 'Y-m-d H:i:s');
            }
            
            foreach ($xmlObj->identificadoresEvts->identificadorEvt as $identificadorEvt) {
                $this->novoEsoEventobxesocial($bxEsocial->getNrbxesocial(), $bxEsocial->getNrevento(), (string)$identificadorEvt->id, (string)$identificadorEvt->nrRec, null);
            }
            
            if ($qtdTotEvts > 50) {
                $this->novoEsoBxesocial(
                    $bxEsocial->getNrestruturam(), $bxEsocial->getNrtpambiente(), $bxEsocial->getNrevento(), $bxEsocial->getDschave(), $dhUltimoEvt, $bxEsocial->getDtfimperiodo(), $bxEsocial->getNrcpftrabalhador(), $bxEsocial->getCdperiodoapuracao()
                );
            }
        } catch(\Exception $e) {
            throw $e;
        }
    }
    
    private function solicitaDownloadEventos($dtsName, $dtsLote, $dsVersao, $eventos) {
        /* Monta e assina o xml do evento */
        $xmlEvento = $this->getXmlEvento($dtsName, $dsVersao, $eventos);
        
        /* Monta o xml do lote de consulta */
        $xmlFactory = new ESocialXmlFactory($dtsLote, $dsVersao);
        $xmlLote = $xmlFactory->buildXml(array('XMLEVENTO' => $xmlEvento));
        
        /* Realiza a transmissão do xml para o webservice do esocial*/
        $transmissor = new ESocialTransmissor($this->tpAmbiente, $this->idOperacao, $this->certKeyFile);
        $xmlResult = $transmissor->transmitirXml($xmlLote);
        
        /* Trata o retorno da consulta */
        $retorno = $this->trataRetornoSolicitacaoDownloadEventos($eventos, $xmlResult);
        
        return $retorno;
    }
    
    private function trataRetornoSolicitacaoDownloadEventos($eventos, $xmlResult) {
        $codRetorno = null;
        $cdErro = null;
        $msg = null;
        
        $xmlObj = simplexml_load_string($xmlResult);

        if ($xmlObj->download->status) {
            $codRetorno = (string) $xmlObj->download->status->cdResposta;
            $msg = (string) $xmlObj->download->status->descResposta;
        } else {
            // Durante os testes quando o esocial não retornoa status é que deu certo
            $codRetorno = '201';
            $msg = 'Download realizado com sucesso';
        }
        
        switch($codRetorno) {
            case '201':
                $cdErro = null;
                break;
            case '301':
            case '307':
            case '308':
            case '309':
            case '310':
            case '402':
            case '403':
            case '404':
            case '405':
            case '406':
            case '407':
            case '409':
            case '410':
            case '411':
            case '417':
                $cdErro = (string) $xmlObj->download->status->cdResposta;
                break;
            default:
                $cdErro = '999';
                $msg = 'Erro do eSocial não identificado';
                break;
        }
        
        $this->addMessage($codRetorno . '-' . $msg);
        
        if (is_null($cdErro)) {
            return true;
        } else {
            return false;
        }
    }
    
    private function retornoSolicDownloadEvtsArquivo($eventos, $xmlObj) {
        $arquivos = array();
        
        foreach ($xmlObj->arquivo as $arquivo) {
            $cdresposta = (string) $arquivo->status->cdResposta;
            $dsresposta = (string) $arquivo->status->descResposta;
            if ($arquivo->evt) {
                $idevento = (string) $arquivo->evt->attributes()->Id;
                $xmlevento = $arquivo->evt->eSocial->asXML();
            } else {
                $idevento = null;
                $xmlevento = null;
            }
            if ($arquivo->rec) {
                $nrrecibo = (string) $arquivo->rec->attributes()->nrRec;
                $xmlrecibo = $arquivo->rec->eSocial->asXML();
            } else {
                $nrrecibo = null;
                $xmlrecibo = null;
            }
            
            $evento = ($eventos[$idevento] ? $eventos[$idevento] : ($eventos[$nrrecibo]));
            
            if (!is_null($evento)) {
                if ($cdresposta == "201") {
                    $eventoBxEsocial = $this->entityManager->getRepository(Repositories::ESO_EVENTOBXESOCIAL)->findOneBy(array('nrorg' => $this->nrorg, 'nreventobxesocial' => $evento['NREVENTOBXESOCIAL']));
                    
                    /* Salva o xml no FileServer */
                    $fileName = $eventoBxEsocial->getNreventobxesocial() . DateUtil::getDataAtual()->format('YmdHis');
                    $xmlPath = $this->saveXmlOnFileServer($xmlevento, $fileName, $fileName);
                    
                    $eventoBxEsocial->setDsdirxmlevento($xmlPath);
                    $eventoBxEsocial->setDtultatu(DateUtil::getDataAtual());
                    $eventoBxEsocial->setCdoperultatu($this->hcmParameters->getCdoperador());
                    $eventoBxEsocial->setNrorgultatu($this->hcmParameters->getNrorgTrabalho());
                    $this->entityManager->persist($eventoBxEsocial);
                    $this->entityManager->flush();
                } else {
                    $this->addMessage($codRetorno . '-' . $msg . ' (Id:' . $idevento . '/Recibo:' . $nrrecibo . ')');
                }
            }
        }
    }
    
    private function validaConsultaIdentificadores($bxEsocial) {
        if ($bxEsocial->getNrtpambiente() != $this->tpAmbiente) {
            throw new \Exception("Tipo de ambiente do evento diferente do tipo de ambiente parametrizado para a estrutura " . $this->nrestruturam);
        }
        if ($bxEsocial->getIdstatus() == 'C') {
            throw new \Exception("O evento já está concluído. Favor atualizar os dados da tela");
        }
    }

    private function trataRetornoEnvioLote($eventosLote, $xmlResult) {
        try {
            $codRetorno = null;
            $idStatus = null;
            $nrProtocolo = null;
            $cdErro = null;
            $msg = null;

            $xmlObj = simplexml_load_string($xmlResult);

            $codRetorno = $xmlObj->retornoEnvioLoteEventos->status->cdResposta;
            switch($xmlObj->retornoEnvioLoteEventos->status->cdResposta) {
                case self::CD_RET_LOTE_201:
                    $idStatus = self::IDSTATUS_PROCESSAMENTO;
                    $cdErro = null;
                    $msg = (string) $xmlObj->retornoEnvioLoteEventos->status->descResposta;
                    $dtRecepcao = (string) $xmlObj->retornoEnvioLoteEventos->dadosRecepcaoLote->dhRecepcao;
                    $nrProtocolo = (string) $xmlObj->retornoEnvioLoteEventos->dadosRecepcaoLote->protocoloEnvio;
                    break;
                case self::CD_RET_LOTE_202:
                    $idStatus = self::IDSTATUS_PROCESSAMENTO;
                    $cdErro = null;
                    $msg = $this->buildMsgErroLote($xmlResult);
                    $dtRecepcao = (string) $xmlObj->retornoEnvioLoteEventos->dadosRecepcaoLote->dhRecepcao;
                    $nrProtocolo = (string) $xmlObj->retornoEnvioLoteEventos->dadosRecepcaoLote->protocoloEnvio;
                    break;
                case self::CD_RET_LOTE_301:
                    $idStatus = self::IDSTATUS_ERRO_ENVIO;
                    $cdErro = (string) $xmlObj->retornoEnvioLoteEventos->status->cdResposta;
                    $msg = null;
                    $dtRecepcao = null;
                    $nrProtocolo = null;
                    break;
                case self::CD_RET_LOTE_401:
                case self::CD_RET_LOTE_402:
                case self::CD_RET_LOTE_403:
                case self::CD_RET_LOTE_404:
                case self::CD_RET_LOTE_405:
                default:
                    $idStatus = self::IDSTATUS_ERRO_ENVIO;
                    $cdErro = (string) $xmlObj->retornoEnvioLoteEventos->status->cdResposta;
                    $msg = $this->buildMsgErroLote($xmlResult);
                    $dtRecepcao = null;
                    $nrProtocolo = null;
                    break;
            }

            if (!is_null($dtRecepcao)) {
                $dtRecepcao = DateUtil::getDataDeString(substr($dtRecepcao,0,10).' '.substr($dtRecepcao,11,8), 'Y-m-d H:i:s');
            }

            foreach ($eventosLote as $evento) {
                $fileName = 'lote' . DateUtil::getDataAtual()->format('YmdHis');

                $envio = $evento['ENVIO'];

                // Salva o arquivo no servidor AWS
                $xmlPath = $this->saveXmlOnFileServer($xmlObj->asXML(), $fileName, $envio->getIdevento(), 'RetornoEnvio');

                $envio->setIdstatus($idStatus);
                $envio->setCderro($cdErro);
                $envio->setDsobservacao(substr($msg,0,4000));
                $envio->setDsobservacao2(null);
                $envio->setDtresposta($dtRecepcao);
                $envio->setNrprotocolo($nrProtocolo);
                $envio->setDsdirxmlretenv($xmlPath);
                $envio->setDtultatu(DateUtil::getDataAtual());
                $envio->setCdoperultatu($this->hcmParameters->getCdoperador());
                $envio->setNrorgultatu($this->hcmParameters->getNrorgTrabalho());
                $this->entityManager->persist($envio);
                $this->entityManager->flush();
                
                // Salva histórico de envio
                $this->salvaHistoricoEnvio($envio);
            }

            return array(
                'COD' => $codRetorno,
                'PRO' => $nrProtocolo,
                'MSG' => $msg,
            );
        } catch(\Exception $e) {
            throw $e;
        }
    }
    
    private function trataRetornoConsultaLote($nrProtocolo, $xmlResult, $idExclusao=false) {
        try {
            $tmpEspera = 0;
            $codRetorno = null;
            $idEvento = null;
            $msg = null;
            $msgRetornoEvento = null;
            $nrRecibo = null;
            $retornoEvento = null;

            $xmlObj = simplexml_load_string($xmlResult);

            $codRetorno = (string)$xmlObj->retornoProcessamentoLoteEventos->status->cdResposta;
            switch($xmlObj->retornoProcessamentoLoteEventos->status->cdResposta) {
                case self::CD_RET_CONS_101:
                    $msg = (string) $xmlObj->retornoProcessamentoLoteEventos->status->descResposta;
                    if ($xmlObj->retornoProcessamentoLoteEventos->status->tempoEstimadoConclusao) {
                        $msg .= ' - Tempo Estimado Conclusão: '. (string) $xmlObj->retornoProcessamentoLoteEventos->status->tempoEstimadoConclusao;
                        $tmpEspera = (string) $xmlObj->retornoProcessamentoLoteEventos->status->tempoEstimadoConclusao;
                    }
                    break;
                case self::CD_RET_CONS_201:
                case self::CD_RET_CONS_202:
                    //$msg = (string) $xmlObj->retornoProcessamentoLoteEventos->status->descResposta;
                    $msg = "";
                    foreach ($xmlObj->retornoProcessamentoLoteEventos->retornoEventos->evento as $evento) {
                        $msgRetornoEvento = '';
                        //Busca o Id do evento
                        foreach ($evento[0]->attributes() as $attName => $attValue) {
                            if ($attName == 'Id') {
                                $idEvento = (string) $attValue;
                            }
                        }

                        //Caso seja o evento de exclusão dos dados do empregador retorna as ocorrências como mensagem pois não existe registro de envio para salvar os dados
                        if ($idExclusao) {
                            $codRetorno = (string) $evento->retornoEvento->eSocial->retornoEvento->processamento->cdResposta;
                            $msg = (string) $evento->retornoEvento->eSocial->retornoEvento->processamento->descResposta;
                            $nrRecibo = (string) $evento->retornoEvento->eSocial->retornoEvento->recibo->nrRecibo;
                            foreach ($evento->retornoEvento->eSocial->retornoEvento->processamento->ocorrencias->ocorrencia as $ocorrencia) {
                                $codRetorno = (string) $ocorrencia->codigo;
                                $msg = (string) $ocorrencia->descricao;
                            }
                        } else {
                            //Trata o retono de processamento do evento e salva os xmls de retorno e totalização
                            if (isset($evento->tot[1])) {
                                // Retorno para os eventos S-1299 e S-1295 em que existem dois totalizadores (S-5011 e S-5012)
                                $eventoTot = clone($evento);
                                unset($eventoTot->retornoEvento);
                                
                                $msgRetornoEvento = $this->trataRetornoProcessamentoEvento($idEvento, $evento->retornoEvento, $eventoTot, $nrProtocolo);
                            } else {
                                // Retorno para eventos com apenas um totalizador
                                $msgRetornoEvento = $this->trataRetornoProcessamentoEvento($idEvento, $evento->retornoEvento, $evento->tot, $nrProtocolo);
                            }
                            if (!empty($msgRetornoEvento) && strpos($msg, $msgRetornoEvento) === false) {
                                $msg .= " " . $msgRetornoEvento;
                            }
                        }
                    }
                    if (empty($msg)) {
                        $msg = (string) $xmlObj->retornoProcessamentoLoteEventos->status->descResposta;
                    } else {
                        $msg = "Lote Processado. " . $msg;
                    }
                    break;
                case self::CD_RET_CONS_301:
                case self::CD_RET_CONS_401:
                case self::CD_RET_CONS_402:
                case self::CD_RET_CONS_403:
                case self::CD_RET_CONS_404:
                case self::CD_RET_CONS_405:
                case self::CD_RET_CONS_501:
                case self::CD_RET_CONS_502:
                case self::CD_RET_CONS_503:
                case self::CD_RET_CONS_504:
                case self::CD_RET_CONS_505:
                default:
                    $msg = $this->buildMsgErroConsulta($xmlResult);
                    
                    /** @var \Teknisa\Model\Gestaopessoas\Entities\EsoEnviocadastros $envio */
                    $envios = $this->entityEnvio->findBy(array('nrprotocolo' => $nrProtocolo, 'nrorg' => $this->hcmParameters->getNrorgTrabalho()));
                    
                    $fileName = 'recibo' . $nrProtocolo . DateUtil::getDataAtual()->format('YmdHis');
                    foreach ($envios as $envio) {

                        // Salva o arquivo no servidor AWS
                        $xmlPath = $this->saveXmlOnFileServer($xmlResult, $fileName, $envio->getIdevento(), 'RetornoProcessamento');
                        
                        $envio->setCderro($codRetorno);
                        //$envio->setDtprocessamento($dtProcessamento);
                        $envio->setDsobservacao(substr($msg,0,4000));
                        //$envio->setDsobservacao2(substr($dsObservacao2,0,1000));
                        $envio->setDsdirxmlretpro($xmlPath);
                        $envio->setDtultatu(DateUtil::getDataAtual());
                        $envio->setCdoperultatu($this->hcmParameters->getCdoperador());
                        $envio->setNrorgultatu($this->hcmParameters->getNrorgTrabalho());
                        $this->entityManager->persist($envio);
                        $this->entityManager->flush();
                    }
                    break;
            }

            return array(
                'COD' => $codRetorno,
                'MSG' => $msg,
                'TMP' => $tmpEspera,
                'REC' => $nrRecibo,
            );
        } catch(\Exception $e) {
            throw $e;
        }
    }

    public function trataRetornoProcessamentoEvento($idEvento, \SimpleXMLElement $xmlRetornoEvento, \SimpleXMLElement $xmlRetornoTotEvento, $nrProtocolo = null) {
        try {
            $msgRetorno = '';
            $dtProcessamento = null;
            $nrRecibo = null;
            $dsObservacao = null;
            $dsObservacao2 = null;
            $dsInfComplem = null;
            $xmlTotPath = null;

            //Salva o xml de retorno do evento
            $fileName = 'evento' . $idEvento . DateUtil::getDataAtual()->format('YmdHis');
            $fileNameTot = 'total' . $idEvento . DateUtil::getDataAtual()->format('YmdHis');
            
            /** @var \Teknisa\Model\Gestaopessoas\Entities\EsoEnviocadastros $envio */
            if(!empty($nrProtocolo)){
                $envio = $this->entityEnvio->findOneBy(array('idevento' => $idEvento, 'nrorg' => $this->hcmParameters->getNrorgTrabalho(), 'nrprotocolo' => $nrProtocolo));
            }else{
                $envio = $this->entityEnvio->findOneBy(array('idevento' => $idEvento, 'nrorg' => $this->hcmParameters->getNrorgTrabalho()));
            }
            if (is_object($envio)) {
                if ($envio->getIdstatus() != self::IDSTATUS_CONCLUIDO_ADVERTENCIA && $envio->getIdstatus() != self::IDSTATUS_CONCLUIDO) {
                    
                    // Salva o arquivo no servidor AWS
                    $xmlPath = $this->saveXmlOnFileServer($xmlRetornoEvento->asXML(), $fileName, $envio->getIdevento(), 'RetornoProcessamento');
                    
                    if ($xmlRetornoTotEvento->asXML()) {
                        
                        // Salva o arquivo no servidor AWS
                        $xmlTotPath = $this->saveXmlOnFileServer($xmlRetornoTotEvento->asXML(), $fileNameTot, $envio->getIdevento(), 'Totalizador');
                        
                        $this->trataRetornoTotalizador($envio, $xmlRetornoTotEvento);
                    }
        
                    $retornoEvento = $xmlRetornoEvento->eSocial->retornoEvento;
        
                    switch($retornoEvento->processamento->cdResposta) {
                        case self::CD_RET_EVEN_201:
                        case self::CD_RET_EVEN_202:
                            if ($retornoEvento->processamento->cdResposta == self::CD_RET_EVEN_202) {
                                $envio->setIdstatus(self::IDSTATUS_CONCLUIDO_ADVERTENCIA);
                                $envio->setCderro((string) $retornoEvento->processamento->cdResposta);
                                $msgErroEvento = $this->buildMsgErroEvento($retornoEvento->processamento);
                                $dsObservacao = $msgErroEvento['msg'];
                                $dsInfComplem = $msgErroEvento['inf'];
                            } else {
                                $envio->setIdstatus(self::IDSTATUS_CONCLUIDO);
                                $dsObservacao = $retornoEvento->processamento->descResposta;
                            }
                            
                            $dtProcessamento = (string) $retornoEvento->processamento->dhProcessamento;
                            $nrRecibo = (string) $retornoEvento->recibo->nrRecibo;
                            
                            /** @var \Teknisa\Model\Gestaopessoas\Entities\EsoEvento $evento */
                            $evento = $this->entityManager->getRepository(Repositories::ESO_EVENTO)->findOneBy(array('nmevento' => $envio->getNmevento(), 'dsversao' => $envio->getDsversao()));
                            
                            $dtmescompetenc = DateUtil::getDataDeString($this->hcmParameters->getDtcompetencia(), DateUtil::FORMATO_BRASILEIRO, true);

                            // Caso seja consulta do evento de fechamento dos periódicos com sucesso de processamento, gera um item de fechamento para a estrutura
                            if ($evento->getNmevento() == 'S-1299') {
                                // Desativa os lançamentos de fechamento e reabertura da estrutura já lançados
                                //$this->unsetEventoEstrutura($envio->getNrorg(), $envio->getNrestruturam(), $dtmescompetenc, $envio->getNrtabela(), 3035);
                                $this->unsetEventoEstrutura($envio->getNrorg(), $envio->getNrestruturam(), $dtmescompetenc, $envio->getNrtabela(), 3036);

                                // Insere um novo lançamento de fechamento para a estrutura
                                $this->setEventoFechamentoEstrutura($envio->getNrorg(), $envio->getNrestruturam(), $envio->getNrtabela());

                                // Desativa o evento S-1298 de reabertura caso já exista
                                $reabertura = $this->entityEnvio->findOneBy(
                                    array('nrorg' => $this->nrorg, 'nrestruturam' => $this->nrestruturam, 'nrtpambiente' => $this->tpAmbiente, 'nmevento' => 'S-1298', 'dtmescompetenc' => $dtmescompetenc, 'nrtabela' => $envio->getNrtabela(), 'idativo' => 'S'),
                                    array('nrenvio' => 'DESC')
                                );
                                if (is_object($reabertura)) {
                                    $reabertura->setIdativo('N');
                                    $reabertura->setNrenvioexcl($envio->getNrenvio());
                                    $this->entityManager->persist($reabertura);
                                }
                            }
                            
                            // Caso seja consulta do evento de reabertura dos periódicos com sucesso de processamento, gera um item de reabertura para a estrutura
                            if ($evento->getNmevento() == 'S-1298') {
                                // Desativa o lançamento de fechamento da estrutura
                                $this->unsetEventoEstrutura($envio->getNrorg(), $envio->getNrestruturam(), $dtmescompetenc, $envio->getNrtabela(), 3035);

                                // Insere um novo lançamento de reabertura para a estrutura
                                $this->setEventoReaberturaEstrutura($envio->getNrorg(), $envio->getNrestruturam(), $envio->getNrtabela());

                                // Desativa o evento S-1298 de fechamento caso já exista
                                $fechamento = $this->entityEnvio->findOneBy(
                                    array('nrorg' => $this->nrorg, 'nrestruturam' => $this->nrestruturam, 'nrtpambiente' => $this->tpAmbiente, 'nmevento' => 'S-1299', 'dtmescompetenc' => $dtmescompetenc, 'nrtabela' => $envio->getNrtabela(), 'idativo' => 'S'),
                                    array('nrenvio' => 'DESC')
                                );
                                if (is_object($fechamento)) {
                                    $fechamento->setIdativo('N');
                                    $fechamento->setNrenvioexcl($envio->getNrenvio());
                                    $this->entityManager->persist($fechamento);
                                }
                            }
                            
                            // Caso seja consulta do evento de admissão preliminiar processado com sucesso, atualiza o número do recibo na pessoa
                            if ($evento->getNmevento() == 'S-2190') {
                                $dtmescompetenc = DateUtil::getDataDeString($this->hcmParameters->getDtcompetencia(), DateUtil::FORMATO_BRASILEIRO, true);
                                $gpePessoah = $this->entityManager->getRepository(Repositories::GPE_PESSOAH)->retornaHistoricoAtualPorPessoa(
                                    $envio->getNrorg(), $envio->getNrchave(), $dtmescompetenc
                                );
                                   if (is_object($gpePessoah)) {
                                    $gpePessoah->setCdreciboadmpre($nrRecibo);
                                    $this->entityManager->persist($gpePessoah);
                                }
                            }
                            
                            // Caso seja consulta do evento de reintegração processado com sucesso, inativa o envio de desligamento informando o envio da reintegração como envio de exclusão
                            if ($evento->getNmevento() == 'S-2298') {
                                $envioRescisao = current($this->entityEnvio->getEnviosEvento(
                                    $envio->getNrorg(), 'S-2299', $envio->getNrestruturam(), $envio->getNrtpambiente(), 'S', $envio->getNrchave(), null, array(), 'N'
                                ));
                                if (is_object($envioRescisao)) {
                                    $envioRescisao->setNrenvioexcl($envio->getNrenvio());
                                    $this->entityManager->persist($envioRescisao);
                                }
                            }
                            
                            break;
                        case self::CD_RET_EVEN_301:
                        case self::CD_RET_EVEN_401:
                        case self::CD_RET_EVEN_402:
                        case self::CD_RET_EVEN_403:
                        case self::CD_RET_EVEN_404:
                        case self::CD_RET_EVEN_405:
                        case self::CD_RET_EVEN_406:
                        case self::CD_RET_EVEN_411:
                        default:
                            $msgRetorno = 'Existem erros/avisos em eventos do lote. Favor verificar. ';
                            $envio->setIdstatus(self::IDSTATUS_ERRO_PROCESSAMENTO);
                            $envio->setCderro((string) $retornoEvento->processamento->cdResposta);
                            $dtProcessamento = (string) $retornoEvento->processamento->dhProcessamento;
                            $msgErroEvento = $this->buildMsgErroEvento($retornoEvento->processamento);
                            $dsObservacao = $msgErroEvento['msg'];
                            $dsInfComplem = $msgErroEvento['inf'];
                            break;
                    }
        
                    if (!is_null($dtProcessamento)) {
                        $dtProcessamento = DateUtil::getDataDeString(substr($dtProcessamento,0,10).' '.substr($dtProcessamento,11,8), 'Y-m-d H:i:s');
                    }
        
                    $envio->setDtprocessamento($dtProcessamento);
                    $envio->setNrrecibo($nrRecibo);
                    $envio->setDsobservacao(substr($dsObservacao,0,4000));
                    $envio->setDsobservacao2(substr((is_null($dsObservacao2) ? $dsInfComplem : $dsObservacao2),0,4000));
                    $envio->setDsdirxmlretpro($xmlPath);
                    if (!is_null($xmlTotPath)) {
                        $envio->setDsdirxmlrettot($xmlTotPath);
                    }
                    $envio->setDtultatu(DateUtil::getDataAtual());
                    $envio->setCdoperultatu($this->hcmParameters->getCdoperador());
                    $envio->setNrorgultatu($this->hcmParameters->getNrorgTrabalho());
                    $this->entityManager->persist($envio);
                    $this->entityManager->flush();
                }
            } else {
                $msgRetorno .= 'Evento não encontrado (ID:'.$idEvento.').';
            }

            return $msgRetorno;
        } catch(\Exception $e) {
            throw $e;
        }
    }
    
    public function trataRetornoTotalizador($envio, \SimpleXMLElement $xmlRetornoTotEvento) {
        try {
            //Trata retorno dos eventos totalizadores
            foreach ($xmlRetornoTotEvento->tot as $totalizador) {
                switch ((string)$totalizador['tipo']) {
                    case 'S5001':
                        $this->trataRetornoTotalizadorS5001($envio, $totalizador);
                        break;
                    case 'S5011':
                        $this->trataRetornoTotalizadorS5011($envio, $totalizador);
                        break;
                }
            }
        } catch(\Exception $e) {
            throw $e;
        }
    }
    
    public function trataRetornoTotalizadorS5001($envio, \SimpleXMLElement $xmlEventoS5001) {
        try {
            $vrCalculado = null;
            $vrDescontado = null;
            $nrEstrutLotacao = null;
            $vrBaseMensal = null;
            $vrBase13 = null;
            $vrValorMensal = null;
            $vrValor13 = null;
            $vrSalFamiliaMensal = null;
            $vrSalFamilia13 = null;
            $vrMaternidadeMensal = null;
            $vrMaternidade13 = null;
            
            if($xmlEventoS5001->eSocial->evtBasesTrab->infoCpCalc){
                $vrCalculado = (float)$xmlEventoS5001->eSocial->evtBasesTrab->infoCpCalc->vrCpSeg;
                $vrDescontado = (float)$xmlEventoS5001->eSocial->evtBasesTrab->infoCpCalc->vrDescSeg;
            }
            if($xmlEventoS5001->eSocial->evtBasesTrab->infoCp){
                $nrEstrutLotacao = (string)$xmlEventoS5001->eSocial->evtBasesTrab->infoCp->ideEstabLot->codLotacao;
                if ($xmlEventoS5001->eSocial->evtBasesTrab->infoCp->ideEstabLot->infoCategIncid->infoBaseCS) {
                    foreach ($xmlEventoS5001->eSocial->evtBasesTrab->infoCp->ideEstabLot->infoCategIncid->infoBaseCS as $infoBaseCS) {
                        //valores mensais
                        if ((string)$infoBaseCS->ind13 == '0') {
                            switch ((string)$infoBaseCS->tpValor) {
                                case '11':
                                case '12':
                                case '13':
                                case '14':
                                case '15':
                                case '16':
                                case '17':
                                case '18':
                                case '19':
                                    $vrBaseMensal = (is_null($vrBaseMensal) ? 0 : $vrBaseMensal) + (float)$infoBaseCS->valor;
                                    break;
                                case '21':
                                case '22':
                                case '23':
                                    $vrValorMensal = (is_null($vrValorMensal) ? 0 : $vrValorMensal) + (float)$infoBaseCS->valor;
                                    break;
                                case '31':
                                    $vrSalFamiliaMensal = (is_null($vrSalFamiliaMensal) ? 0 : $vrSalFamiliaMensal) + (float)$infoBaseCS->valor;
                                    break;
                                case '32':
                                    $vrMaternidadeMensal = (is_null($vrMaternidadeMensal) ? 0 : $vrMaternidadeMensal) + (float)$infoBaseCS->valor;
                                    break;
                            }
                        }
                        
                        //valores 13
                        if ((string)$infoBaseCS->ind13 == '1') {
                            switch ((string)$infoBaseCS->tpValor) {
                                case '11':
                                case '12':
                                case '13':
                                case '14':
                                case '15':
                                case '16':
                                case '17':
                                case '18':
                                case '19':
                                    $vrBase13 = (is_null($vrBase13) ? 0 : $vrBase13) + (float)$infoBaseCS->valor;
                                    break;
                                case '21':
                                case '22':
                                case '23':
                                    $vrValor13 = (is_null($vrValor13) ? 0 : $vrValor13) + (float)$infoBaseCS->valor;
                                    break;
                                case '31':
                                    $vrSalFamilia13 = (is_null($vrSalFamilia13) ? 0 : $vrSalFamilia13) + (float)$infoBaseCS->valor;
                                    break;
                                case '32':
                                    $vrMaternidade13 = (is_null($vrMaternidade13) ? 0 : $vrMaternidade13) + (float)$infoBaseCS->valor;
                                    break;
                            }
                        }
                    }
                }
            }
            
            $envio->setVrcalculado($vrCalculado);
            $envio->setVrdescontado($vrDescontado);
            $envio->setNrestrutlotacao($nrEstrutLotacao);
            $envio->setVrbasemensal($vrBaseMensal);
            $envio->setVrbase13($vrBase13);
            $envio->setVrvalormensal($vrValorMensal);
            $envio->setVrvalor13($vrValor13);
            $envio->setVrsalfamilia($vrSalFamiliaMensal);
            $envio->setVrsalfamilia13($vrSalFamilia13);
            $envio->setVrvalormatermensal($vrMaternidadeMensal);
            $envio->setVrvalormater13($vrMaternidade13);
            $this->entityManager->persist($envio);
            $this->entityManager->flush();
        } catch(\Exception $e) {
            throw $e;
        }
    }
    
    public function trataRetornoTotalizadorS5011($envio, \SimpleXMLElement $xmlEventoS5011) {
        try {
            $vrCalculado = null;
            $vrDescontado = null;
            $nrEstrutLotacao = null;
            $vrBaseMensal = null;
            $vrBase13 = null;
            $vrValorMensal = null;
            $vrValor13 = null;
            $vrSalFamiliaMensal = null;
            $vrSalFamilia13 = null;
            $vrMaternidadeMensal = null;
            $vrMaternidade13 = null;
            
            $vrCalculado = (float)$xmlEventoS5011->eSocial->evtCS->infoCS->infoCPSeg->vrCpSeg;
            $vrDescontado = (float)$xmlEventoS5011->eSocial->evtCS->infoCS->infoCPSeg->vrDescCP;
            /*$nrEstrutLotacao = (string)$xmlEventoS5011->eSocial->evtCS->infoCS->ideEstab->codLotacao;
            if ($xmlEventoS5011->eSocial->evtCS->infoCS->ideEstab->infoCategIncid->infoBaseCS) {
                foreach ($xmlEventoS5011->eSocial->evtCS->infoCS->ideEstab->infoCategIncid->infoBaseCS as $infoBaseCS) {
                    //valores mensais
                    if ((string)$infoBaseCS->ind13 == '0') {
                        switch ((string)$infoBaseCS->tpValor) {
                            case '11':
                                $vrBaseMensal = (float)$infoBaseCS->valor;
                                break;
                            case '21':
                                $vrValorMensal = (float)$infoBaseCS->valor;
                                break;
                            case '31':
                                $vrSalFamiliaMensal = (float)$infoBaseCS->valor;
                                break;
                            case '32':
                                $vrMaternidadeMensal = (float)$infoBaseCS->valor;
                                break;
                        }
                    }
                    
                    //valores 13
                    if ((string)$infoBaseCS->ind13 == '1') {
                        switch ((string)$infoBaseCS->tpValor) {
                            case '11':
                                $vrBase13 = (float)$infoBaseCS->valor;
                                break;
                            case '21':
                                $vrValor13 = (float)$infoBaseCS->valor;
                                break;
                            case '31':
                                $vrSalFamilia13 = (float)$infoBaseCS->valor;
                                break;
                            case '32':
                                $vrMaternidade13 = (float)$infoBaseCS->valor;
                                break;
                        }
                    }
                }
            }*/
            
            $envio->setVrcalculado($vrCalculado);
            $envio->setVrdescontado($vrDescontado);
            /*$envio->setNrestrutlotacao($nrEstrutLotacao);
            $envio->setVrbasemensal($vrBaseMensal);
            $envio->setVrbase13($vrBase13);
            $envio->setVrvalormensal($vrValorMensal);
            $envio->setVrvalor13($vrValor13);
            $envio->setVrsalfamilia($vrSalFamiliaMensal);
            $envio->setVrsalfamilia13($vrSalFamilia13);
            $envio->setVrvalormatermensal($vrMaternidadeMensal);
            $envio->setVrvalormater13($vrMaternidade13);*/
            $this->entityManager->persist($envio);
            $this->entityManager->flush();
        } catch(\Exception $e) {
            throw $e;
        }
    }
    
    /**
     * @param $processamento
     * @return string
     * @throws \Exception
     */
    public function buildMsgErroEvento(\SimpleXMLElement $processamento) {
        try {
            $msg = '';
            $msg .= $processamento->cdResposta . '-' . $processamento->descResposta;
            $inf = '';

            if ($processamento->ocorrencias->ocorrencia) {
                foreach ($processamento->ocorrencias->ocorrencia as $ocorrencia) {
                    $msg .= "\n[" . (isset($this->_tipoOcorRetorno[(string)$ocorrencia->tipo]) ? $this->_tipoOcorRetorno[(string)$ocorrencia->tipo] : (string)$ocorrencia->tipo) . '] ' . $ocorrencia->codigo . '-' . $ocorrencia->descricao;
                    if ($ocorrencia->localizacao) {
                        $msg .= " (" . $ocorrencia->localizacao . ")";
                    }
                    $inf .= $this->verificaInfComplementar((string)$ocorrencia->codigo, (string)$ocorrencia->descricao);
                }
            }

            return array(
                'msg' => $msg,
                'inf' => ($inf == '' ? null : $inf),
            );
        } catch(\Exception $e) {
            throw $e;
        }
    }
    
    public function verificaInfComplementar($codigo, $descricao) {
        try {
            $infComplementar = $this->entityManager->getRepository(Repositories::ESO_INFCOMPLEMENTAR)->findOneBy(array('cderro' => str_pad($codigo, 4, "0", STR_PAD_LEFT)));

            if (is_object($infComplementar)) {
                if ($infComplementar->getDsinfcomplementar()) {
                    return $infComplementar->getCderro() . "-" . $infComplementar->getDsinfcomplementar() . "\n";
                } else {
                    return '';
                }
            } else {
                return '';
            }
        } catch(\Exception $e) {
            throw $e;
        }
    }

    /**
     * @param $xmlResponse
     * @return string
     * @throws \Exception
     */
    public function buildMsgErroLote($xmlResponse) {
        try {
            $xmlObj = simplexml_load_string($xmlResponse);

            $msg = '';
            $msg .= $xmlObj->retornoEnvioLoteEventos->status->cdResposta . '-' . $xmlObj->retornoEnvioLoteEventos->status->descResposta;

            foreach ($xmlObj->retornoEnvioLoteEventos->status->ocorrencias->ocorrencia as $ocorrencia) {
                $msg .= "\n[" . (isset($this->_tipoOcorRetorno[(string)$ocorrencia->tipo]) ? $this->_tipoOcorRetorno[(string)$ocorrencia->tipo] : (string)$ocorrencia->tipo) . '] ' . $ocorrencia->codigo . '-' . $ocorrencia->descricao;
                if ($ocorrencia->localizacao) {
                    $msg .= " (" . $ocorrencia->localizacao . ")";
                }
            }

            return $msg;
        } catch(\Exception $e) {
            throw $e;
        }
    }
    
    /**
     * @param $envio
     * @param $evento
     * @return \Teknisa\Model\Gestaopessoas\Entities\EsoHistoricoenvio
     * @throws \Exception
     */
    private function salvaHistoricoEnvio($envio) {
        try {
            /** @var \Teknisa\Model\Gestaopessoas\Entities\EsoHistoricoenvio $esoHitoricoenvio */
            $esoHitoricoenvio = Repositories::ESO_HISTORICOENVIO;
            
            $evento = $this->getEsoEvento($envio->getNmevento(), $envio->getDsversao());
            
            $historico = new $esoHitoricoenvio();
            $historico->setNrorg($envio->getNrorg());
            $historico->setNrenvio($envio->getNrenvio());
            $historico->setNrevento($evento->getNrevento());
            $historico->setIdevento($envio->getIdevento());
            $historico->setNrtpambiente($envio->getNrtpambiente());
            $historico->setCdprotocolo($envio->getNrprotocolo());
            $historico->setDtenvio($envio->getDtenvio());
            $historico->setDtresposta($envio->getDtresposta());
            $historico->setDsobservacao($envio->getDsobservacao());
            $historico->setDsdirxmlenv($envio->getDsdirxmlenv());
            $historico->setDsdirxmlretenv($envio->getDsdirxmlretenv());
            $historico->setDtinclusao(DateUtil::getDataAtual());
            $historico->setCdoperinclusao($this->hcmParameters->getCdoperador());
            $historico->setNrorginclusao($this->hcmParameters->getNrorgTrabalho());
            $historico->setNrhistoricoenvio(\HCM\Util\NovoCodigo::geraCodigo($this->hcmParameters->getNrorgTrabalho(), 'ESO_HISTORICOENVIO', 1, 12, true));
            
            $this->entityManager->persist($historico);

            return $historico;
        } catch(\Exception $e) {
            throw $e;
        }
    }
    
    public function setEventoFechamentoEstrutura($nrorg, $nrestruturam, $nrtipomoviment) {
        try {
            $nrcalculo = $this->hcmParameters->determinaNrCalculoFolha($nrtipomoviment);
            
            /** @var \Teknisa\Model\Gestaopessoas\Entities\FpaItecalcestrut $envio */
            $fpaItecalcestrut = Repositories::FPA_ITECALCESTRUT;
            
            $item = new $fpaItecalcestrut();
            $item->setNrorg($nrorg);
            $item->setNrtipoestrutura($this->hcmParameters->getTipoEstruturaLegalOrg());
            $item->setNrestruturam($nrestruturam);
            $item->setNreventom(3035);
            $item->setNrcalculofolha($nrcalculo);
            $item->setDtocorrencia(DateUtil::getDataDeString($this->hcmParameters->getDtcompetencia(), DateUtil::FORMATO_BRASILEIRO, true));
            $item->setVrlancestrutura(0);
            $item->setIdativo('S');
            $item->setDtinclusao(DateUtil::getDataAtual());
            $item->setCdoperinclusao($this->hcmParameters->getCdoperador());
            $item->setNrorginclusao($this->hcmParameters->getNrorgTrabalho());
            $item->setNritecalcestrut(\HCM\Util\NovoCodigo::geraCodigo($this->hcmParameters->getNrorgTrabalho(), 'FPA_ITECALCESTRUT', 1, 12, true));
            
            $this->entityManager->persist($item);
        } catch(\Exception $e) {
            throw $e;
        }
    }
    
    public function setEventoReaberturaEstrutura($nrorg, $nrestruturam, $nrtipomoviment) {
        try {
            $nrcalculo = $this->hcmParameters->determinaNrCalculoFolha($nrtipomoviment);
            
            /** @var \Teknisa\Model\Gestaopessoas\Entities\FpaItecalcestrut $envio */
            $fpaItecalcestrut = Repositories::FPA_ITECALCESTRUT;
            
            $item = new $fpaItecalcestrut();
            $item->setNrorg($nrorg);
            $item->setNrtipoestrutura($this->hcmParameters->getTipoEstruturaLegalOrg());
            $item->setNrestruturam($nrestruturam);
            $item->setNreventom(3036);
            $item->setNrcalculofolha($nrcalculo);
            $item->setDtocorrencia(DateUtil::getDataDeString($this->hcmParameters->getDtcompetencia(), DateUtil::FORMATO_BRASILEIRO, true));
            $item->setVrlancestrutura(0);
            $item->setIdativo('S');
            $item->setDtinclusao(DateUtil::getDataAtual());
            $item->setCdoperinclusao($this->hcmParameters->getCdoperador());
            $item->setNrorginclusao($this->hcmParameters->getNrorgTrabalho());
            $item->setNritecalcestrut(\HCM\Util\NovoCodigo::geraCodigo($this->hcmParameters->getNrorgTrabalho(), 'FPA_ITECALCESTRUT', 1, 12, true));
            
            $this->entityManager->persist($item);
        } catch(\Exception $e) {
            throw $e;
        }
    }
    
    public function unsetEventoEstrutura($nrorg, $nrestruturam, $dtmescompetenc, $nrtipomoviment, $nreventom) {
        try {
            $nrcalculo = $this->hcmParameters->determinaNrCalculoFolha($nrtipomoviment);
            
            // Verifica se a estrutura possui lançamento do evento
            $lancEstrutura = $this->entityManager->getRepository(Repositories::FPA_ITECALCESTRUT)->findOneBy(array(
                'nrorg' => $nrorg, 'nrestruturam' => $nrestruturam, 'dtocorrencia' => $dtmescompetenc, 'nreventom' => $nreventom, 'idativo' => 'S', 'nrcalculofolha' => $nrcalculo
            ));
            
            if (is_object($lancEstrutura)) {
                $lancEstrutura->setIdativo('N');
                $this->entityManager->persist($lancEstrutura);
                $this->entityManager->flush();
                return true;
            }
            
            return false;
        } catch(\Exception $e) {
            throw $e;
        }
    }
    
    private function enviarEvento($row, $consultar=false) {
        $this->idOperacao = self::IDTRANSENVIO;
        
        $msgValidacao = array();
        $msgRetorno = '';
        
        $envio = $this->entityEnvio->find(array('nrenvio' => $row['NRENVIO'], 'nrorg' => $this->hcmParameters->getNrorgTrabalho()));
        
        if (!is_object($envio)) {
            throw new \Exception('Registro de envio não encontrado. Favor atualizar os dados da tela e tentar novamente.', Messages::ERROR_MESSAGE_CODE);
        }
        
        $msgProcesso = MsgProcesso::getInstance($this->hcmParameters->getNrorg(), $this->hcmParameters->getCdoperador(), $row['NRENVIO']);
        
        // Valida se o status do evento no banco de dados já está em processamento ou concluído caso a tela não esteja atualizada e o evento seja enviado novamente
        $idStatus = true;
        if (($envio->getIdstatus() == 'M' || $envio->getIdstatus() == 'C' || $envio->getIdstatus() == 'A' || ($this->idGrupo == self::IDGRUPO_CADASTRO ? false : !is_null($envio->getNrenvioexcl()))) &&
            ($envio->getCderro() != '301' && $envio->getCderro() != '69')) {
            $idStatus = false;
            throw new \Exception('O evento já está em processamento ou concluído. Favor atualizar os dados da tela.', Messages::ERROR_MESSAGE_CODE);
        }
        
        // monta o xml do evento
        $dtsName = $row['NMEVENTO'];
        $dsVersao = str_replace('.','-',$row['DSVERSAO']);
        $dados = $row;
        
        // Função para geração de ID's
        if (is_null($dados['IDEVENTO']) || !$dados['IDEVENTO']) {
            $idEvento = $this->geraIdEvento($dados['NRESTRUTURAM'], $dados['NRENVIO']);
            $dados['IDEVENTO'] = $idEvento;
        } else {
            $idEvento = $dados['IDEVENTO'];
        }
        $dados['ID'] = $idEvento;
        
        // Verifica se é um evento de retificação e busca o recibo de processamento do envio a ser retificado
        if ($dados['IDOPERACAO'] == 'R') {
            $envioOrig = $this->entityEnvio->findOneBy(array('nrorg' => $dados['NRORG'], 'nrenvioexcl' => $dados['NRENVIO']));
            if ($envioOrig) {
                $dados['NRRECIBOORIG'] = $envioOrig->getNrrecibo();
            }
        }
        
        $xmlEvento = $this->getXmlEvento($dtsName, $dsVersao, $dados, true, true);
        
        $nmXmlEvento = $idEvento . DateUtil::getDataAtual()->format('YmdHis');
        
        // Salva o arquivo no servidor AWS
        $xmlPath = $this->saveXmlOnFileServer($xmlEvento, $nmXmlEvento, $idEvento, 'Envio');
        
        // Salva os dados de envio
        $envio->setIdevento($idEvento);
        $envio->setDtenvio(DateUtil::getDataAtual());
        $envio->setDsdirxmlenv($xmlPath);
        $envio->setDtultatu(DateUtil::getDataAtual());
        $envio->setCdoperultatu($this->hcmParameters->getCdoperador());
        $envio->setNrorgultatu($this->hcmParameters->getNrorgTrabalho());
        $this->entityManager->persist($envio);
        
        $xmlEvento = array(
            "NRENVIO" => $dados['NRENVIO'],
            "IDEVENTO" => $idEvento,
            "NMXMLEVENTO" => $nmXmlEvento,
            "XMLEVENTO" => $xmlEvento,
            "ENVIO" => $envio,
        );
        

        if ($xmlEvento) {
            $eventosLote[] = $xmlEvento;
        }
        
        foreach ($msgProcesso->getMessages() as $msg) {
            $msgValidacao[] = array(
                'NMEVENTO' => $row['NMEVENTO'],
                'NRCHAVE' => $row['NRCHAVE'],
                'MSG' => $msg['MSG'],
                'TYPE' => $msg['TYPE'],
            );
        }
        $msgProcesso->clear();
        unset($msgProcesso);
        
        if (empty($msgValidacao) && empty($eventosLote)) {
            // caso não existam nem eventos no lote nem pendências
            throw new \Exception("HCM09000_ESO_LOTE_VAZIO_ERROR");
        } else if (empty($msgValidacao)) {
            // se existem eventos no lote, são enviados
            // eventos com pendências não são enviados (não estão no array de lote)
            $xmlLote = $this->getXmlLote($this->idGrupo, $dsVersao, $eventosLote);
            
            /* Realiza a transmissão do xml para o webservice do esocial*/
            $transmissor = new ESocialTransmissor($this->tpAmbiente, $this->idOperacao, $this->certKeyFile);
            $xmlResult = $transmissor->transmitirXml($xmlLote);
            
            $retorno = $this->trataRetornoEnvioLote($eventosLote, $xmlResult);
            $this->entityManager->flush();
            
            $msgRetorno = $retorno['COD'] . '-' . $retorno['MSG'];
            
            if ($consultar) {
                if ($retorno['COD'] == '201') {
                    $nrProtocolo = $retorno['PRO']; // Protocolo do lote
                    
                    $retornoConsulta = $this->consultarProtocolo($dsVersao, $nrProtocolo);
                    $msgRetornoConsulta = $retornoConsulta['COD'] . '-' . $retornoConsulta['MSG'];
                    
                    /*if ($retorno['COD'] == '101') {
                        // Caso o evento ainda esteja em processamento aguarda o tempo indicado e faz nova consulta
                        sleep($tmpEsperaConsulta ? (int)$tmpEsperaConsulta : 3);
                        $xmlConsulta = $this->eSocialService->getXmlConsulta($nrProtocolo, $dsVersao);
                        $retorno = $this->eSocialService->consultarEvento($nrProtocolo, $xmlConsulta);
                    }*/
                    
                    $msgRetorno = $msgRetorno.' / Consulta: '.$msgRetornoConsulta;
                }
            }
        } else {
            foreach ($msgValidacao as $msg) {
                $this->addMessage($msg);
            }
        }
        
        return $msgRetorno;
    }
    
    private function retificarEvento($row) {
        $this->idOperacao = self::IDTRANSENVIO;
        
        $msgValidacao = array();
        $msgRetorno = '';
        
        $envioOrig = $this->entityEnvio->find(array('nrenvio' => $row['NRENVIO'], 'nrorg' => $this->hcmParameters->getNrorgTrabalho()));
        
        if (!is_object($envioOrig)) {
            throw new \Exception('Registro de envio não encontrado. Favor atualizar os dados da tela e tentar novamente.', Messages::ERROR_MESSAGE_CODE);
        }
        
        if ($envioOrig->getNrenvioexcl()) {
            throw new \Exception('Registro já foi retificado ou excluído. Favor atualizar os dados da tela e tentar novamente.', Messages::ERROR_MESSAGE_CODE);
        }
        
        $envioRetif = $this->novoEsoNaoPeriodico(
            $envioOrig->getNrestruturam(), $envioOrig->getNrvinculom(), $envioOrig->getNmevento(), $envioOrig->getDsversao(), $envioOrig->getNrtpambiente(), $envioOrig->getNrchave(), $envioOrig->getDtmescompetenc(), 'R'
        );
        
        $msgProcesso = MsgProcesso::getInstance($this->hcmParameters->getNrorg(), $this->hcmParameters->getCdoperador(), $row['NRENVIO']);
        
        // monta o xml do evento
        $dtsName = $row['NMEVENTO'];
        $dsVersao = str_replace('.','-',$row['DSVERSAO']);
        
        // Função para geração de ID's
        $idEvento = $this->geraIdEvento($envioRetif->getNrestruturam(), $envioRetif->getNrenvio());
        $envioRetif->setIdevento($idEvento);
        
        $dados = $envioRetif->toArray();
        
        $dados['ID'] = $idEvento;
        $dados['IDOPERACAO'] = "R";
        $dados['INDRETIF'] = 2;
        $dados['NRRECIBOORIG'] = $envioOrig->getNrrecibo();
        
        $xmlEvento = $this->getXmlEvento($dtsName, $dsVersao, $dados, true, true);
        
        $nmXmlEvento = $idEvento . DateUtil::getDataAtual()->format('YmdHis');
        
        // Salva o arquivo no servidor AWS
        $xmlPath = $this->saveXmlOnFileServer($xmlEvento, $nmXmlEvento, $idEvento, 'Envio');
        
        // Salva os dados de envio
        $envioRetif->setIdevento($idEvento);
        $envioRetif->setDtenvio(DateUtil::getDataAtual());
        $envioRetif->setDsdirxmlenv($xmlPath);
        $envioRetif->setDtultatu(DateUtil::getDataAtual());
        $envioRetif->setCdoperultatu($this->hcmParameters->getCdoperador());
        $envioRetif->setNrorgultatu($this->hcmParameters->getNrorgTrabalho());
        $this->entityManager->persist($envioRetif);
        
        $envioOrig->setNrenvioexcl($envioRetif->getNrenvio());
        $this->entityManager->persist($envioOrig);
        
        $xmlEvento = array(
            "NRENVIO" => $dados['NRENVIO'],
            "IDEVENTO" => $idEvento,
            "NMXMLEVENTO" => $nmXmlEvento,
            "XMLEVENTO" => $xmlEvento,
            "ENVIO" => $envioRetif,
        );
        

        if ($xmlEvento) {
            $eventosLote[] = $xmlEvento;
        }
        
        foreach ($msgProcesso->getMessages() as $msg) {
            $msgValidacao[] = array(
                'NMEVENTO' => $row['NMEVENTO'],
                'NRCHAVE' => $row['NRCHAVE'],
                'MSG' => $msg['MSG'],
                'TYPE' => $msg['TYPE'],
            );
        }
        $msgProcesso->clear();
        unset($msgProcesso);
        
        if (empty($msgValidacao) && empty($eventosLote)) {
            // caso não existam nem eventos no lote nem pendências
            throw new \Exception("HCM09000_ESO_LOTE_VAZIO_ERROR");
        } else if (empty($msgValidacao)) {
            // se existem eventos no lote, são enviados
            // eventos com pendências não são enviados (não estão no array de lote)
            $xmlLote = $this->getXmlLote($this->idGrupo, $dsVersao, $eventosLote);
            
            /* Realiza a transmissão do xml para o webservice do esocial*/
            $transmissor = new ESocialTransmissor($this->tpAmbiente, $this->idOperacao, $this->certKeyFile);
            $xmlResult = $transmissor->transmitirXml($xmlLote);
            
            $retorno = $this->trataRetornoEnvioLote($eventosLote, $xmlResult);
            
            $msgRetorno = $retorno['COD'] . '-' . $retorno['MSG'];
            
            if ($retorno['COD'] == '201') {
                $this->entityManager->flush();
                //$this->entityManager->commit();
                //$this->entityManager->beginTransaction();
        
                $nrProtocolo = $retorno['PRO']; // Protocolo do lote
                
                $retornoConsulta = $this->consultarProtocolo($dsVersao, $nrProtocolo);
                $msgRetornoConsulta = $retornoConsulta['COD'] . '-' . $retornoConsulta['MSG'];
                
                /*if ($retorno['COD'] == '101') {
                    // Caso o evento ainda esteja em processamento aguarda o tempo indicado e faz nova consulta
                    sleep($tmpEsperaConsulta ? (int)$tmpEsperaConsulta : 3);
                    $xmlConsulta = $this->eSocialService->getXmlConsulta($nrProtocolo, $dsVersao);
                    $retorno = $this->eSocialService->consultarEvento($nrProtocolo, $xmlConsulta);
                }*/
                
                $msgRetorno = $msgRetorno.' / Consulta da Retificação: '.$msgRetornoConsulta;
            }
        } else {
            foreach ($msgValidacao as $msg) {
                $this->addMessage($msg);
            }
        }
        
        return $msgRetorno;
    }
    
    private function excluirEvento($row) {
        $this->idOperacao = self::IDTRANSENVIO;
        
        $msgValidacao = array();
        $msgRetorno = '';
        
        /** @var \Teknisa\Model\Gestaopessoas\Entities\EsoEvento $esoeventoexclusao */
        $esoeventoexclusao = $this->entityManager->getRepository(Repositories::ESO_EVENTO)->findOneBy(array(
            'nmevento' => 'S-3000', 'dsversao' => $this->dsVersao
        ));
        
        if (!is_object($esoeventoexclusao)) {
            throw new \Exception('Registro de evento S-3000 não encontrado. Favor atualizar os dados da tela e tentar novamente.', Messages::ERROR_MESSAGE_CODE);
        }
        
        $envioOrig = $this->entityEnvio->find(array('nrenvio' => $row['NRENVIO'], 'nrorg' => $this->hcmParameters->getNrorgTrabalho()));
        
        if (!is_object($envioOrig)) {
            throw new \Exception('Registro de envio não encontrado. Favor atualizar os dados da tela e tentar novamente.', Messages::ERROR_MESSAGE_CODE);
        }
        
        if ($envioOrig->getNrenvioexcl()) {
            throw new \Exception('Registro já foi retificado ou excluído. Favor atualizar os dados da tela e tentar novamente.', Messages::ERROR_MESSAGE_CODE);
        }
        
        $envioExcl = $this->novoEsoNaoPeriodico(
            $envioOrig->getNrestruturam(), $envioOrig->getNrvinculom(), $esoeventoexclusao->getNmevento(), $this->dsVersao, $envioOrig->getNrtpambiente(), $envioOrig->getNrchave(), $envioOrig->getDtmescompetenc(), 'I'
        );
        
        $msgProcesso = MsgProcesso::getInstance($this->hcmParameters->getNrorg(), $this->hcmParameters->getCdoperador(), $row['NRENVIO']);
        
        // monta o xml do evento
        $dtsName = $esoeventoexclusao->getNmevento();
        $dsVersao = str_replace('.','-',$this->dsVersao);
        $idGrupo = $esoeventoexclusao->getIdgrupo(); //@todo criar função para pegar o grupo de acordo com o evento em questão.. no antigo o grupo vinha do front pois o datasource da tela tinha join com evento
        
        // Função para geração de ID's
        $idEvento = $this->geraIdEvento($envioExcl->getNrestruturam(), $envioExcl->getNrenvio());
        $envioExcl->setIdevento($idEvento);
        
        $dados = $envioExcl->toArray();
        
        $vinculo = $this->entityManager->getRepository(Repositories::GPE_VINCULOM)->findOneBy(array('nrorg' => $dados['NRORG'], 'nrvinculom' => $dados['NRVINCULOM']));
        $nrIdeTrabalhador = is_object($vinculo) ? $vinculo->getNrpessoa() : null;
        
        $dados['ID'] = $idEvento;
        $dados['IDOPERACAO'] = "I";
        $dados['NRENVIOORIG'] = $envioOrig->getNrenvio();
        $dados['NRRECIBOORIG'] = $envioOrig->getNrrecibo();
        $dados['NRIDETRABALHADOR'] = $nrIdeTrabalhador;
        
        $xmlEvento = $this->getXmlEvento($dtsName, $dsVersao, $dados, true, true);
        
        $nmXmlEvento = $idEvento . DateUtil::getDataAtual()->format('YmdHis');
        
        // Salva o arquivo no servidor AWS
        $xmlPath = $this->saveXmlOnFileServer($xmlEvento, $nmXmlEvento, $idEvento, 'Envio');
        
        // Salva os dados de envio
        $envioExcl->setIdevento($idEvento);
        $envioExcl->setDtenvio(DateUtil::getDataAtual());
        $envioExcl->setDsdirxmlenv($xmlPath);
        $envioExcl->setDtultatu(DateUtil::getDataAtual());
        $envioExcl->setCdoperultatu($this->hcmParameters->getCdoperador());
        $envioExcl->setNrorgultatu($this->hcmParameters->getNrorgTrabalho());
        $this->entityManager->persist($envioExcl);
        
        $envioOrig->setNrenvioexcl($envioExcl->getNrenvio());
        $this->entityManager->persist($envioOrig);
        
        $xmlEvento = array(
            "NRENVIO" => $dados['NRENVIO'],
            "IDEVENTO" => $idEvento,
            "NMXMLEVENTO" => $nmXmlEvento,
            "XMLEVENTO" => $xmlEvento,
            "ENVIO" => $envioExcl,
        );
        

        if ($xmlEvento) {
            $eventosLote[] = $xmlEvento;
        }
        
        foreach ($msgProcesso->getMessages() as $msg) {
            $msgValidacao[] = array(
                'NMEVENTO' => $row['NMEVENTO'],
                'NRCHAVE' => $row['NRCHAVE'],
                'MSG' => $msg['MSG'],
                'TYPE' => $msg['TYPE'],
            );
        }
        $msgProcesso->clear();
        unset($msgProcesso);
        
        if (empty($msgValidacao) && empty($eventosLote)) {
            // caso não existam nem eventos no lote nem pendências
            throw new \Exception("HCM09000_ESO_LOTE_VAZIO_ERROR");
        } else if (empty($msgValidacao)) {
            // se existem eventos no lote, são enviados
            // eventos com pendências não são enviados (não estão no array de lote)
            $xmlLote = $this->getXmlLote($idGrupo, $dsVersao, $eventosLote);
            
            /* Realiza a transmissão do xml para o webservice do esocial*/
            $transmissor = new ESocialTransmissor($this->tpAmbiente, $this->idOperacao, $this->certKeyFile);
            $xmlResult = $transmissor->transmitirXml($xmlLote);
            
            $retorno = $this->trataRetornoEnvioLote($eventosLote, $xmlResult);
            
            $msgRetorno = $retorno['COD'] . '-' . $retorno['MSG'];
            
            if ($retorno['COD'] == '201') {
                $this->entityManager->flush();
                //$this->entityManager->commit();
                //$this->entityManager->beginTransaction();
        
                $nrProtocolo = $retorno['PRO']; // Protocolo do lote
                
                $retornoConsulta = $this->consultarProtocolo($dsVersao, $nrProtocolo);
                $msgRetornoConsulta = $retornoConsulta['COD'] . '-' . $retornoConsulta['MSG'];
                
                /*if ($retorno['COD'] == '101') {
                    // Caso o evento ainda esteja em processamento aguarda o tempo indicado e faz nova consulta
                    sleep($tmpEsperaConsulta ? (int)$tmpEsperaConsulta : 3);
                    $xmlConsulta = $this->eSocialService->getXmlConsulta($nrProtocolo, $dsVersao);
                    $retorno = $this->eSocialService->consultarEvento($nrProtocolo, $xmlConsulta);
                }*/
                
                $msgRetorno = $msgRetorno.' / Consulta da Exclusão: '.$msgRetornoConsulta;
            }
        } else {
            foreach ($msgValidacao as $msg) {
                $this->addMessage($msg);
            }
        }
        
        return $msgRetorno;
    }
    
    private function envioAutomaticoEvento($nmevento, $idstatus) {
        $this->idOperacao = self::IDTRANSENVIO;
        
        $msgValidacao = array();
        $msgRetorno = '';
        
        $envios = $this->entityEnvio->findBy(
            array('nrorg' => $this->nrorg, 'nrtpambiente' => $this->tpAmbiente, 'nrestruturam' => $this->nrestruturam, 'idativo' => 'S', 'nmevento' => $nmevento, 'idstatus' => $idstatus)
        );
        
        if (empty($envios)) {
            throw new \Exception('Não foram encontrados eventos para enviar.');
        }
        
        foreach($envios as $envio) {
            $msgProcesso = MsgProcesso::getInstance($this->nrorg, $this->hcmParameters->getCdoperador(), $envio->getNrenvio());
            
            // Valida se o status do evento no banco de dados já está em processamento ou concluído caso a tela não esteja atualizada e o evento seja enviado novamente
            $idStatus = true;
            if (($envio->getIdstatus() == 'M' || $envio->getIdstatus() == 'C' || $envio->getIdstatus() == 'A' || !is_null($envio->getNrenvioexcl())) &&
                ($envio->getCderro() != '301' && $envio->getCderro() != '69')) {
                continue;
            }
            
            // monta o xml do evento
            $dtsName = $envio->getNmevento();
            $dsVersao = str_replace('.','-',$envio->getDsversao());
            $dados = $envio->toArray();
            
            // Função para geração de ID's
            if (is_null($dados['IDEVENTO']) || !$dados['IDEVENTO']) {
                $idEvento = $this->geraIdEvento($dados['NRESTRUTURAM'], $dados['NRENVIO']);
                $dados['IDEVENTO'] = $idEvento;
            } else {
                $idEvento = $dados['IDEVENTO'];
            }
            $dados['ID'] = $idEvento;
            
            // Verifica se é um evento de retificação e busca o recibo de processamento do envio a ser retificado
            if ($dados['IDOPERACAO'] == 'R') {
                $envioOrig = $this->entityEnvio->findOneBy(array('nrorg' => $dados['NRORG'], 'nrenvioexcl' => $dados['NRENVIO']));
                if ($envioOrig) {
                    $dados['NRRECIBOORIG'] = $envioOrig->getNrrecibo();
                }
            }
            
            $xmlEvento = $this->getXmlEvento($dtsName, $dsVersao, $dados, true, true);
            
            $nmXmlEvento = $idEvento . DateUtil::getDataAtual()->format('YmdHis');
            
            // Salva o arquivo no servidor AWS
            $xmlPath = $this->saveXmlOnFileServer($xmlEvento, $nmXmlEvento, $idEvento, 'Envio');
            
            // Salva os dados de envio
            $envio->setIdevento($idEvento);
            $envio->setDtenvio(DateUtil::getDataAtual());
            $envio->setDsdirxmlenv($xmlPath);
            $envio->setDtultatu(DateUtil::getDataAtual());
            $envio->setCdoperultatu($this->hcmParameters->getCdoperador());
            $envio->setNrorgultatu($this->hcmParameters->getNrorgTrabalho());
            $this->entityManager->persist($envio);
            
            $xmlEvento = array(
                "NRENVIO" => $dados['NRENVIO'],
                "IDEVENTO" => $idEvento,
                "NMXMLEVENTO" => $nmXmlEvento,
                "XMLEVENTO" => $xmlEvento,
                "ENVIO" => $envio,
            );
            
    
            if ($xmlEvento) {
                $eventosLote[] = $xmlEvento;
            }
            
            foreach ($msgProcesso->getMessages() as $msg) {
                $msgValidacao[] = array(
                    'NMEVENTO' => $row['NMEVENTO'],
                    'NRCHAVE' => $row['NRCHAVE'],
                    'MSG' => $msg['MSG'],
                    'TYPE' => $msg['TYPE'],
                );
            }
            $msgProcesso->clear();
            unset($msgProcesso);
        }
        
        if (empty($msgValidacao) && empty($eventosLote)) {
            // caso não existam nem eventos no lote nem pendências
            throw new \Exception("HCM09000_ESO_LOTE_VAZIO_ERROR");
        } else if (empty($msgValidacao)) {
            // se existem eventos no lote, são enviados
            // eventos com pendências não são enviados (não estão no array de lote)
            $xmlLote = $this->getXmlLote($this->idGrupo, $dsVersao, $eventosLote);
            
            /* Realiza a transmissão do xml para o webservice do esocial*/
            $transmissor = new ESocialTransmissor($this->tpAmbiente, $this->idOperacao, $this->certKeyFile);
            $xmlResult = $transmissor->transmitirXml($xmlLote);
            
            $retorno = $this->trataRetornoEnvioLote($eventosLote, $xmlResult);
            $this->entityManager->flush();
            
            $msgRetorno = $retorno['COD'] . '-' . $retorno['MSG'];
        } else {
            foreach ($msgValidacao as $msg) {
                $this->addMessage($msg);
            }
        }
        
        return $msgRetorno;
    }
    
    private function consultarProtocolo($dsVersao, $nrProtocolo, $idExclusao=false) {
        $this->idOperacao = self::IDTRANSCONSU;
        
        $msgValidacao = array();
        $msgRetorno = '';
        
        $xmlConsulta = $this->getXmlConsulta($nrProtocolo, $dsVersao);
            
        /* Realiza a transmissão do xml para o webservice do esocial*/
        $transmissor = new ESocialTransmissor($this->tpAmbiente, $this->idOperacao, $this->certKeyFile);
        $xmlResult = $transmissor->transmitirXml($xmlConsulta);
        
        $retorno = $this->trataRetornoConsultaLote($nrProtocolo, $xmlResult, $idExclusao);
        
        return $retorno;
    }
    
    private function consultaAutomaticaEvento($nmevento, $idstatus) {
        $this->idOperacao = self::IDTRANSCONSU;
        
        $msgRetorno = array();
        
        $envios = $this->entityEnvio->findBy(
            array('nrorg' => $this->nrorg, 'nrtpambiente' => $this->tpAmbiente, 'nrestruturam' => $this->nrestruturam, 'idativo' => 'S', 'nmevento' => $nmevento, 'idstatus' => $idstatus)
        );
        
        if (empty($envios)) {
            throw new \Exception('Não foram encontrados eventos para consultar.');
        }
        
        // Armazena os diferentes recibos existentes nos envios filtrados
        $arrayProtocolos = array();
        foreach ($envios as $envio) {
            if (!$envio->getNrenvioexcl() && !isset($arrayProtocolos[$envio->getDsversao()][$envio->getNrprotocolo()])) {
                $arrayProtocolos[$envio->getDsversao()][$envio->getNrprotocolo()] = $envio->getNrprotocolo();
            }
        }
        
        foreach($arrayProtocolos as $dsversao => $protocolos) {
            sort($protocolos);
            foreach($protocolos as $nrprotocolo) {
                $xmlConsulta = $this->getXmlConsulta($nrprotocolo, str_replace('.','-',$dsversao));
            
                /* Realiza a transmissão do xml para o webservice do esocial*/
                $transmissor = new ESocialTransmissor($this->tpAmbiente, $this->idOperacao, $this->certKeyFile);
                $xmlResult = $transmissor->transmitirXml($xmlConsulta);
                
                $retorno = $this->trataRetornoConsultaLote($nrprotocolo, $xmlResult);
                $msgRetorno[] = $retorno['COD'] . '-' . $retorno['MSG'];
            }
        }
        
        return implode(' / ',$msgRetorno);
    }
    
    private function enviarExclusaoEmpregador($row) {
        $this->idOperacao = self::IDTRANSENVIO;
        
        $msgValidacao = array();
        $msgRetorno = '';
        
        /** @var \Teknisa\Model\Gestaopessoas\Entities\EsoEvento $esoeventoexclusao */
        $esoeventoexclusao = $this->entityManager->getRepository(Repositories::ESO_EVENTO)->findOneBy(array(
            'nmevento' => 'S-1000', 'dsversao' => $this->dsVersao
        ));
        
        if (!is_object($esoeventoexclusao)) {
            throw new \Exception('Registro de evento S-1000 não encontrado. Favor atualizar os dados da tela e tentar novamente.', Messages::ERROR_MESSAGE_CODE);
        }
        
        $envioExcl = $this->novoEsoCadastro(
            $row['NRESTRUTURAM'], $esoeventoexclusao->getNmevento(), $this->dsVersao, $this->tpAmbiente, $row['NRESTRUTURAM'], $this->dtmescompetenc.' 00:00:00', 'E'
        );
        
        $msgProcesso = MsgProcesso::getInstance($this->hcmParameters->getNrorg(), $this->hcmParameters->getCdoperador(), $envioExcl->getNrenvio());
        
        // monta o xml do evento
        $dtsName = $esoeventoexclusao->getNmevento();
        $dsVersao = str_replace('.','-',$this->dsVersao);
        $idGrupo = $esoeventoexclusao->getIdgrupo(); //@todo criar função para pegar o grupo de acordo com o evento em questão.. no antigo o grupo vinha do front pois o datasource da tela tinha join com evento
        
        // Função para geração de ID's
        $idEvento = $this->geraIdEvento($envioExcl->getNrestruturam(), $envioExcl->getNrenvio());
        $envioExcl->setIdevento($idEvento);
        
        $dados = $envioExcl->toArray();
        
        $dados['ID'] = $idEvento;
        $dados['IDOPERACAO'] = 'I';
        
        $xmlEvento = $this->getXmlEvento($dtsName, $dsVersao, $dados, true, true, true);
        
        $nmXmlEvento = $idEvento . DateUtil::getDataAtual()->format('YmdHis');
        
        // Salva o arquivo no servidor AWS
        $xmlPath = $this->saveXmlOnFileServer($xmlEvento, $nmXmlEvento, $idEvento, 'Envio');
        
        // Salva os dados de envio
        $envioExcl->setIdevento($idEvento);
        $envioExcl->setDtenvio(DateUtil::getDataAtual());
        $envioExcl->setDsdirxmlenv($xmlPath);
        $envioExcl->setDtultatu(DateUtil::getDataAtual());
        $envioExcl->setCdoperultatu($this->hcmParameters->getCdoperador());
        $envioExcl->setNrorgultatu($this->hcmParameters->getNrorgTrabalho());
        $envioExcl->setDsobservacao2('RemoverEmpregadorDaBaseDeDadosDaProducaoRestrita');
        $this->entityManager->persist($envioExcl);
        
        $xmlEvento = array(
            "NRENVIO" => $dados['NRENVIO'],
            "IDEVENTO" => $idEvento,
            "NMXMLEVENTO" => $nmXmlEvento,
            "XMLEVENTO" => $xmlEvento,
            "ENVIO" => $envioExcl,
        );
        

        if ($xmlEvento) {
            $eventosLote[] = $xmlEvento;
        }
        
        foreach ($msgProcesso->getMessages() as $msg) {
            $msgValidacao[] = array(
                'NMEVENTO' => $row['NMEVENTO'],
                'NRCHAVE' => $row['NRCHAVE'],
                'MSG' => $msg['MSG'],
                'TYPE' => $msg['TYPE'],
            );
        }
        $msgProcesso->clear();
        unset($msgProcesso);
        
        if (empty($msgValidacao) && empty($eventosLote)) {
            // caso não existam nem eventos no lote nem pendências
            throw new \Exception("HCM09000_ESO_LOTE_VAZIO_ERROR");
        } else if (empty($msgValidacao)) {
            // se existem eventos no lote, são enviados
            // eventos com pendências não são enviados (não estão no array de lote)
            $xmlLote = $this->getXmlLote($idGrupo, $dsVersao, $eventosLote);
            
            /* Realiza a transmissão do xml para o webservice do esocial*/
            $transmissor = new ESocialTransmissor($this->tpAmbiente, $this->idOperacao, $this->certKeyFile);
            $xmlResult = $transmissor->transmitirXml($xmlLote);
            
            $retorno = $this->trataRetornoEnvioLote($eventosLote, $xmlResult);
            
            $msgRetorno = $retorno['COD'] . '-' . $retorno['MSG'];
            
            if ($retorno['COD'] == '201') {
                $this->entityManager->flush();
        
                $nrProtocolo = $retorno['PRO']; // Protocolo do lote
                
                $retornoConsulta = $this->consultarProtocolo($dsVersao, $nrProtocolo, true);
                $msgRetornoConsulta = $retornoConsulta['COD'] . '-' . $retornoConsulta['MSG'];
                
                /*if ($retorno['COD'] == '101') {
                    // Caso o evento ainda esteja em processamento aguarda o tempo indicado e faz nova consulta
                    sleep($tmpEsperaConsulta ? (int)$tmpEsperaConsulta : 3);
                    $xmlConsulta = $this->eSocialService->getXmlConsulta($nrProtocolo, $dsVersao);
                    $retorno = $this->eSocialService->consultarEvento($nrProtocolo, $xmlConsulta);
                }*/
                
                if ($retornoConsulta['COD'] == '1012') {
                    // Remove todos eventos de produção restrita do empregador
                    $this->entityManager->getRepository(Repositories::ESO_CADASTRO)->deleteEnviosExclusaoEmpregador($envioExcl->getNrorg(), $envioExcl->getNrestruturam());
                }
                
                $msgRetorno = $msgRetorno.' / Consulta da Exclusão: '.$msgRetornoConsulta.' / '.$envioExcl->getDsobservacao().' ['.$retornoConsulta['COD'].']';
            }
            
            $envioExcl->setIdativo('N');
            $this->entityManager->flush();
        } else {
            foreach ($msgValidacao as $msg) {
                $this->addMessage($msg);
            }
        }
        
        return $msgRetorno;
    }
    
    public function getBxesocial($nrbxesocial) {
        $bxEsocial = $this->entityManager->getRepository(Repositories::ESO_BXESOCIAL)
            ->findOneBy(array('nrorg' => $this->nrorg, 'nrbxesocial' => $nrbxesocial));
        
        return $bxEsocial;
    }
    
    public function getEventobxesocial($nreventobxesocial) {
        $eventoBxEsocial = $this->entityManager->getRepository(Repositories::ESO_EVENTOBXESOCIAL)
            ->findOneBy(array('nrorg' => $this->nrorg, 'nreventobxesocial' => $nreventobxesocial));
        
        return $eventoBxEsocial;
    }

    public function novoEsoBxesocial($nrestruturam, $nrtpambiente, $nrevento, $dschave, $dtiniperiodo, $dtfimperiodo, $nrcpftrabalhador, $cdperiodoapuracao, $idstatus='P') {
        try {
            $esoBxesocial = Repositories::ESO_BXESOCIAL;
            
            /** @var \Teknisa\Model\Gestaopessoas\Entities\EsoBxesocial $novo */
            $novo = new $esoBxesocial();
            $novo->setNrestruturam($nrestruturam);
            $novo->setNrtpambiente($nrtpambiente);
            $novo->setNrevento($nrevento);
            $novo->setDschave($dschave);
            $novo->setDtiniperiodo($dtiniperiodo);
            $novo->setDtfimperiodo($dtfimperiodo);
            $novo->setNrcpftrabalhador($nrcpftrabalhador);
            $novo->setCdperiodoapuracao($cdperiodoapuracao);
            $novo->setIdstatus($idstatus);
            $novo->setDtinclusao(DateUtil::getDataAtual());
            $novo->setCdoperinclusao($this->hcmParameters->getCdoperador());
            $novo->setNrorginclusao($this->hcmParameters->getNrorgTrabalho());
            $novo->setNrorg($this->hcmParameters->getNrorgTrabalho());
            $novo->setNrbxesocial(\HCM\Util\NovoCodigo::geraCodigo($this->hcmParameters->getNrorgTrabalho(), 'ESO_BXESOCIAL', 1, 12, true));
            
            $this->entityManager->persist($novo);
            
            return $novo;
        } catch(\Exception $e) {
            throw $e;
        }
    }

    public function novoEsoEventobxesocial($nrbxesocial, $nrevento, $idevento, $nrrecibo, $dsdirxmlevento, $idativo='S') {
        try {
            $esoEventobxesocial = Repositories::ESO_EVENTOBXESOCIAL;
            
            /** @var \Teknisa\Model\Gestaopessoas\Entities\EsoEventobxesocial $novo */
            $novo = new $esoEventobxesocial();
            $novo->setNrbxesocial($nrbxesocial);
            $novo->setNrevento($nrevento);
            $novo->setIdevento($idevento);
            $novo->setNrrecibo($nrrecibo);
            $novo->setDsdirxmlevento($dsdirxmlevento);
            $novo->setIdativo($idativo);
            $novo->setDtinclusao(DateUtil::getDataAtual());
            $novo->setCdoperinclusao($this->hcmParameters->getCdoperador());
            $novo->setNrorginclusao($this->hcmParameters->getNrorgTrabalho());
            $novo->setNrorg($this->hcmParameters->getNrorgTrabalho());
            $novo->setNreventobxesocial(\HCM\Util\NovoCodigo::geraCodigo($this->hcmParameters->getNrorgTrabalho(), 'ESO_EVENTOBXESOCIAL', 1, 12, true));
            
            $this->entityManager->persist($novo);
            
            return $novo;
        } catch(\Exception $e) {
            throw $e;
        }
    }
    
    public function consultarIdentificadoresEventosEmpregador($nrbxesocial) {
        $this->idOperacao = self::IDCONSUIDEMPR;
        
        $bxEsocial = $this->getBxesocial($nrbxesocial);
        
        $this->validaConsultaIdentificadores($bxEsocial);
        
        if ($bxEsocial->getNrevento()) {
            $esoEvento = $this->entityManager->getRepository(Repositories::ESO_EVENTO)->findOneBy(array('nrevento' => $bxEsocial->getNrevento()));
            $filtros['TPEVT'] = (is_object($esoEvento) ? $esoEvento->getNmevento() : null);
        } else {
            $filtros['TPEVT'] = null;
        }
        $filtros['PERAPUR'] = $bxEsocial->getCdperiodoapuracao();
        
        $dtsName = 'ConsultaIdentificadoresEventosEmpregador';
        $dtsLote = 'ConsultarIdentificadoresEventosEmpregador';
        $dsVersao = '1_0_0';
        
        return $this->consultarIdentificadoresEventos($bxEsocial, $dtsName, $dtsLote, $dsVersao, $filtros);
    }
    
    public function consultarIdentificadoresEventosTabela($nrbxesocial) {
        $this->idOperacao = self::IDCONSUIDTABE;
        
        $bxEsocial = $this->getBxesocial($nrbxesocial);
        
        $this->validaConsultaIdentificadores($bxEsocial);
        
        if ($bxEsocial->getNrevento()) {
            $esoEvento = $this->entityManager->getRepository(Repositories::ESO_EVENTO)->findOneBy(array('nrevento' => $bxEsocial->getNrevento()));
            $filtros['TPEVT'] = (is_object($esoEvento) ? $esoEvento->getNmevento() : null);
        } else {
            $filtros['TPEVT'] = null;
        }
        $filtros['CHEVT'] = $bxEsocial->getDschave();
        $filtros['DTINI'] = $bxEsocial->getDtiniperiodo();
        $filtros['DTFIM'] = $bxEsocial->getDtfimperiodo();
        
        $dtsName = 'ConsultaIdentificadoresEventosTabela';
        $dtsLote = 'ConsultarIdentificadoresEventosTabela';
        $dsVersao = '1_0_0';
        
        return $this->consultarIdentificadoresEventos($bxEsocial, $dtsName, $dtsLote, $dsVersao, $filtros);
    }
    
    public function consultarIdentificadoresEventosTrabalhador($nrbxesocial) {
        $this->idOperacao = self::IDCONSUIDTRAB;
        
        $bxEsocial = $this->getBxesocial($nrbxesocial);
        
        $this->validaConsultaIdentificadores($bxEsocial);
        
        $filtros['CPFTRAB'] = $bxEsocial->getNrcpftrabalhador();
        $filtros['DTINI'] = $bxEsocial->getDtiniperiodo();
        $filtros['DTFIM'] = $bxEsocial->getDtfimperiodo();
        
        $dtsName = 'ConsultaIdentificadoresEventosTrabalhador';
        $dtsLote = 'ConsultarIdentificadoresEventosTrabalhador';
        $dsVersao = '1_0_0';
        
        return $this->consultarIdentificadoresEventos($bxEsocial, $dtsName, $dtsLote, $dsVersao, $filtros);
    }
    
    public function downloadEventosPorId($nrbxesocial, $eventos) {
        $this->idOperacao = self::IDDOWNLOADID;
        
        $bxEsocial = $this->getBxesocial($nrbxesocial);
        
        foreach ($eventos as $evento) {
            $ideventos[$evento['IDEVENTO']] = array(
                'NREVENTOBXESOCIAL' => $evento['NREVENTOBXESOCIAL'],
                'IDEVENTO' => $evento['IDEVENTO'],
            );
        }
        
        $dtsName = 'SolicitacaoDownloadEventosPorId';
        $dtsLote = 'SolicitarDownloadEventosPorId';
        $dsVersao = '1_0_0';
        
        return $this->solicitaDownloadEventos($dtsName, $dtsLote, $dsVersao, $ideventos);
    }
    
    public function downloadEventosPorNrRecibo($nrbxesocial, $eventos) {
        $this->idOperacao = self::IDDOWNLOADREC;
        
        $bxEsocial = $this->getBxesocial($nrbxesocial);
        
        foreach ($eventos as $evento) {
            $recibos[$evento['NRRECIBO']] = array(
                'NREVENTOBXESOCIAL' => $evento['NREVENTOBXESOCIAL'],
                'NRRECIBO' => $evento['NRRECIBO'],
            );
        }
        
        $dtsName = 'SolicitacaoDownloadEventosPorNrRecibo';
        $dtsLote = 'SolicitarDownloadEventosPorNrRecibo';
        $dsVersao = '1_0_0';
        
        return $this->solicitaDownloadEventos($dtsName, $dtsLote, $dsVersao, $recibos);
    }
    
    public function novoEsoCadastro($nrestruturam, $nmevento, $dsversao, $nrtpambiente, $nrchave, $dtmescompetenc, $idoperacao) {
        try {
            $esoCadastro = Repositories::ESO_CADASTRO;
            
            $novo = new $esoCadastro();
            $novo->setNrestruturam($nrestruturam);
            $novo->setNmevento($nmevento);
            $novo->setDsversao($dsversao);
            $novo->setNrtpambiente($nrtpambiente);
            $novo->setNrchave($nrchave);
            $novo->setDtmescompetenc(DateUtil::getDataDeString($dtmescompetenc, DateUtil::FORMATO_BRASILEIRO_DATAHORA, true));
            $novo->setIdoperacao($idoperacao);
            $novo->setIdstatus('P');
            $novo->setNrorg($this->hcmParameters->getNrorgTrabalho());
            $novo->setDtinclusao(DateUtil::getDataAtual());
            $novo->setCdoperinclusao($this->hcmParameters->getCdoperador());
            $novo->setNrorginclusao($this->hcmParameters->getNrorgTrabalho());
            $this->entityManager->persist($novo);
            
            
            $nrenvio = \HCM\Util\NovoCodigo::geraCodigo($this->hcmParameters->getNrorgTrabalho(), 'ESO_CADASTRO', 1, 12, true);
            $novo->setNrenvio($nrenvio);
            
            return $novo;
        } catch(\Exception $e) {
            throw $e;
        }
    }
    
    public function novoEsoNaoPeriodico($nrestruturam, $nrvinculom, $nmevento, $dsversao, $nrtpambiente, $nrchave, $dtmescompetenc, $idoperacao, $nrtabela=null) {
        try {
            $esoNaoperiodico = Repositories::ESO_NAOPERIODICO;
            
            $novo = new $esoNaoperiodico();
            $novo->setNrestruturam($nrestruturam);
            $novo->setNrvinculom($nrvinculom);
            $novo->setNmevento($nmevento);
            $novo->setDsversao($dsversao);
            $novo->setNrtpambiente($nrtpambiente);
            $novo->setNrchave($nrchave);
            $novo->setDtmescompetenc(DateUtil::getDataDeString($dtmescompetenc, DateUtil::FORMATO_BRASILEIRO_DATAHORA, true));
            $novo->setIdoperacao($idoperacao);
            $novo->setIdstatus('P');
            $novo->setNrorg($this->hcmParameters->getNrorgTrabalho());
            $novo->setDtinclusao(DateUtil::getDataAtual());
            $novo->setCdoperinclusao($this->hcmParameters->getCdoperador());
            $novo->setNrorginclusao($this->hcmParameters->getNrorgTrabalho());
            $novo->setNrtabela($nrtabela);
            
            $nrenvio = \HCM\Util\NovoCodigo::geraCodigo($this->hcmParameters->getNrorgTrabalho(), 'ESO_NAOPERIODICO', 1, 12, true);
            $novo->setNrenvio($nrenvio);
            
            $this->entityManager->persist($novo);
            
            return $novo;
        } catch(\Exception $e) {
            throw $e;
        }
    }
    
    public function novoEsoPeriodico($nrestruturam, $nrpessoa, $nmevento, $dsversao, $nrtpambiente, $nrchave, $nrtabela, $dtmescompetenc, $idoperacao) {
        try {
            $esoPeriodico = Repositories::ESO_PERIODICO;
            
            $novo = new $esoPeriodico();
            $novo->setNrestruturam($nrestruturam);
            $novo->setNrpessoa($nrpessoa);
            $novo->setNmevento($nmevento);
            $novo->setDsversao($dsversao);
            $novo->setNrtpambiente($nrtpambiente);
            $novo->setNrchave($nrchave);
            $novo->setNrtabela($nrtabela);
            $novo->setDtmescompetenc(DateUtil::getDataDeString($dtmescompetenc, DateUtil::FORMATO_BRASILEIRO_DATAHORA, true));
            $novo->setIdoperacao($idoperacao);
            $novo->setIdstatus('P');
            $novo->setNrorg($this->hcmParameters->getNrorgTrabalho());
            $novo->setDtinclusao(DateUtil::getDataAtual());
            $novo->setCdoperinclusao($this->hcmParameters->getCdoperador());
            $novo->setNrorginclusao($this->hcmParameters->getNrorgTrabalho());
            
            $nrenvio = \HCM\Util\NovoCodigo::geraCodigo($this->hcmParameters->getNrorgTrabalho(), 'ESO_PERIODICO', 1, 12, true);
            $novo->setNrenvio($nrenvio);
            
            $this->entityManager->persist($novo);
            
            return $novo;
        } catch(\Exception $e) {
            throw $e;
        }
    }
    
    /*-----Cadastros-----*/
    public function enviarEventoCadastro($row) {
        $this->idGrupo = self::IDGRUPO_CADASTRO;
        $this->entityEnvio = $this->entityManager->getRepository(Repositories::ESO_CADASTRO);
        
        return $this->enviarEvento($row, true);
    }
    
    public function retificarEventoCadastro($row) {
        $this->idGrupo = self::IDGRUPO_CADASTRO;
        $this->entityEnvio = $this->entityManager->getRepository(Repositories::ESO_CADASTRO);
        
        return $this->retificarEvento($row);
    }
    
    public function excluirEventoCadastro($row) {
        $this->idGrupo = self::IDGRUPO_CADASTRO;
        $this->entityEnvio = $this->entityManager->getRepository(Repositories::ESO_CADASTRO);
        
        return $this->excluirEvento($row);
    }
    
    public function consultarProtocoloCadastro($row) {
        $this->idGrupo = self::IDGRUPO_CADASTRO;
        $this->entityEnvio = $this->entityManager->getRepository(Repositories::ESO_CADASTRO);
        
        $dsVersao = str_replace('.','-',$row['DSVERSAO']);
        $nrProtocolo = $row['NRPROTOCOLO'];
        
        $retornoConsulta = $this->consultarProtocolo($dsVersao, $nrProtocolo);
        
        return $retornoConsulta['COD'] . '-' . $retornoConsulta['MSG'];
    }
    
    public function envioAutomaticoCadastro($nmevento, $idstatus) {
        $this->idGrupo = self::IDGRUPO_CADASTRO;
        $this->entityEnvio = $this->entityManager->getRepository(Repositories::ESO_CADASTRO);
        
        return $this->envioAutomaticoEvento($nmevento, $idstatus);
    }
    
    public function consultaAutomaticaCadastro($nmevento, $idstatus) {
        $this->idGrupo = self::IDGRUPO_CADASTRO;
        $this->entityEnvio = $this->entityManager->getRepository(Repositories::ESO_CADASTRO);
        
        return $this->consultaAutomaticaEvento($nmevento, $idstatus);
    }
    
    public function exclusaoEmpregador($row) {
        if ($this->tpAmbiente != 2) {
            throw new \Exception("A exclusão do empregador só é possível no ambiente de produção restrita");
        }
        
        $this->idGrupo = self::IDGRUPO_CADASTRO;
        $this->entityEnvio = $this->entityManager->getRepository(Repositories::ESO_CADASTRO);
        
        return $this->enviarExclusaoEmpregador($row);
    }
    
    /*-----Não Periodicos-----*/
    public function enviarEventoNaoPeriodico($row) {
        $this->idGrupo = self::IDGRUPO_NAOPERIODICO;
        $this->entityEnvio = $this->entityManager->getRepository(Repositories::ESO_NAOPERIODICO);
        
        return $this->enviarEvento($row, true);
    }
    
    public function retificarEventoNaoPeriodico($row) {
        $this->idGrupo = self::IDGRUPO_NAOPERIODICO;
        $this->entityEnvio = $this->entityManager->getRepository(Repositories::ESO_NAOPERIODICO);
        
        return $this->retificarEvento($row);
    }
    
    public function excluirEventoNaoPeriodico($row) {
        $this->idGrupo = self::IDGRUPO_NAOPERIODICO;
        $this->entityEnvio = $this->entityManager->getRepository(Repositories::ESO_NAOPERIODICO);
        
        return $this->excluirEvento($row);
    }
    
    public function consultarProtocoloNaoPeriodico($row) {
        $this->idGrupo = self::IDGRUPO_NAOPERIODICO;
        $this->entityEnvio = $this->entityManager->getRepository(Repositories::ESO_NAOPERIODICO);
        
        $dsVersao = str_replace('.','-',$row['DSVERSAO']);
        $nrProtocolo = $row['NRPROTOCOLO'];
        
        $retornoConsulta = $this->consultarProtocolo($dsVersao, $nrProtocolo);
        
        return $retornoConsulta['COD'] . '-' . $retornoConsulta['MSG'];
    }
    
    public function envioAutomaticoNaoPeriodico($nmevento, $idstatus) {
        $this->idGrupo = self::IDGRUPO_NAOPERIODICO;
        $this->entityEnvio = $this->entityManager->getRepository(Repositories::ESO_NAOPERIODICO);
        
        return $this->envioAutomaticoEvento($nmevento, $idstatus);
    }
    
    public function consultaAutomaticaNaoPeriodico($nmevento, $idstatus) {
        $this->idGrupo = self::IDGRUPO_NAOPERIODICO;
        $this->entityEnvio = $this->entityManager->getRepository(Repositories::ESO_NAOPERIODICO);
        
        return $this->consultaAutomaticaEvento($nmevento, $idstatus);
    }
    
    /*-----Periodicos-----*/
    public function enviarEventoPeriodico($row) {
        $this->idGrupo = self::IDGRUPO_PERIODICO;
        $this->entityEnvio = $this->entityManager->getRepository(Repositories::ESO_PERIODICO);
        
        return $this->enviarEvento($row, true);
    }
    
    public function retificarEventoPeriodico($row) {
        $this->idGrupo = self::IDGRUPO_PERIODICO;
        $this->entityEnvio = $this->entityManager->getRepository(Repositories::ESO_PERIODICO);
        
        return $this->retificarEvento($row);
    }
    
    public function excluirEventoPeriodico($row) {
        $this->idGrupo = self::IDGRUPO_PERIODICO;
        $this->entityEnvio = $this->entityManager->getRepository(Repositories::ESO_PERIODICO);
        
        return $this->excluirEvento($row);
    }
    
    public function consultarProtocoloPeriodico($row) {
        $this->idGrupo = self::IDGRUPO_PERIODICO;
        $this->entityEnvio = $this->entityManager->getRepository(Repositories::ESO_PERIODICO);
        
        $dsVersao = str_replace('.','-',$row['DSVERSAO']);
        $nrProtocolo = $row['NRPROTOCOLO'];
        
        $retornoConsulta = $this->consultarProtocolo($dsVersao, $nrProtocolo);
        
        return $retornoConsulta['COD'] . '-' . $retornoConsulta['MSG'];
    }
    
    public function envioAutomaticoPeriodico($nmevento, $idstatus) {
        $this->idGrupo = self::IDGRUPO_PERIODICO;
        $this->entityEnvio = $this->entityManager->getRepository(Repositories::ESO_PERIODICO);
        
        return $this->envioAutomaticoEvento($nmevento, $idstatus);
    }
    
    public function consultaAutomaticaPeriodico($nmevento, $idstatus) {
        $this->idGrupo = self::IDGRUPO_PERIODICO;
        $this->entityEnvio = $this->entityManager->getRepository(Repositories::ESO_PERIODICO);
        
        return $this->consultaAutomaticaEvento($nmevento, $idstatus);
    }

    public function fechamentoEsoSimplificado($row) {
        $this->idGrupo = self::IDGRUPO_PERIODICO;
        $this->entityEnvio = $this->entityManager->getRepository(Repositories::ESO_PERIODICO);

        $dtmescompetenc = DateUtil::getDataDeString($this->hcmParameters->getDtCompetencia(), DateUtil::FORMATO_BRASILEIRO, true);

        $fechamento = $this->entityEnvio->findOneBy(
            array('nrorg' => $this->nrorg, 'nrestruturam' => $this->nrestruturam, 'nrtpambiente' => $this->tpAmbiente, 'nmevento' => 'S-1299', 'dtmescompetenc' => $dtmescompetenc, 'nrtabela' => $row['NRTABELA'], 'idativo' => 'S'),
            array('nrenvio' => 'DESC')
        );

        if(!$fechamento) {
            $fechamento = $this->novoEsoPeriodico($this->nrestruturam, null, 'S-1299', $this->dsVersao, $this->tpAmbiente, $this->nrestruturam, $row['NRTABELA'], $dtmescompetenc, 'I');
            $this->entityManager->flush();
        }

        if ($fechamento->getIdstatus() == self::IDSTATUS_PENDENTE || $fechamento->getIdstatus() == self::IDSTATUS_ERRO_ENVIO || $fechamento->getIdstatus() == self::IDSTATUS_ERRO_PROCESSAMENTO) {
            $retorno = $this->enviarEvento($fechamento->toArray(), true);
        } else if ($fechamento->getIdstatus() == self::IDSTATUS_PROCESSAMENTO) {
            $dsVersao = str_replace('.','-',$fechamento->getDsversao());
            $retornoConsulta = $this->consultarProtocolo($dsVersao, $fechamento->getNrprotocolo());
            $retorno = $retornoConsulta['COD'] . '-' . $retornoConsulta['MSG'];
        } else {
            throw new \Exception("O evento de fechamento já foi enviado. Favor atualizar os dados da tela");
        }
        
        return $retorno;
    }

    public function reaberturaEsoSimplificado($row) {
        $this->idGrupo = self::IDGRUPO_PERIODICO;
        $this->entityEnvio = $this->entityManager->getRepository(Repositories::ESO_PERIODICO);

        $dtmescompetenc = DateUtil::getDataDeString($this->hcmParameters->getDtCompetencia(), DateUtil::FORMATO_BRASILEIRO, true);

        $reabertura = $this->entityEnvio->findOneBy(
            array('nrorg' => $this->nrorg, 'nrestruturam' => $this->nrestruturam, 'nrtpambiente' => $this->tpAmbiente, 'nmevento' => 'S-1298', 'dtmescompetenc' => $dtmescompetenc, 'nrtabela' => $row['NRTABELA'], 'idativo' => 'S'),
            array('nrenvio' => 'DESC')
        );

        if(!$reabertura) {
            $reabertura = $this->novoEsoPeriodico($this->nrestruturam, null, 'S-1298', $this->dsVersao, $this->tpAmbiente, $this->nrestruturam, $row['NRTABELA'], $dtmescompetenc, 'I');
            $this->entityManager->flush();
        }

        if ($reabertura->getIdstatus() == self::IDSTATUS_PENDENTE || $reabertura->getIdstatus() == self::IDSTATUS_ERRO_ENVIO || $reabertura->getIdstatus() == self::IDSTATUS_ERRO_PROCESSAMENTO) {
            $retorno = $this->enviarEvento($reabertura->toArray(), true);
        } else if ($reabertura->getIdstatus() == self::IDSTATUS_PROCESSAMENTO) {
            $dsVersao = str_replace('.','-',$reabertura->getDsversao());
            $retornoConsulta = $this->consultarProtocolo($dsVersao, $reabertura->getNrprotocolo());
            $retorno = $retornoConsulta['COD'] . '-' . $retornoConsulta['MSG'];
        } else {
            throw new \Exception("O evento de reabertura já foi enviado. Favor atualizar os dados da tela");
        }
        
        return $retorno;
    }
    
    public function setXmlFileToDataSet($xmlString) {
        try {
            $json = $this->parseXmlToJson($xmlString);
            $dataSet = $this->translateESocialJson($json);
            return $dataSet;
        } catch(\Exception $e) {
            throw $e;
        }
    }
    
    public function parseXmlToJson($xmlFileContents) {
        try {
    		$xmlFileContents = str_replace(array("\n", "\r", "\t"), '', $xmlFileContents);
    		$xmlFileContents = trim(str_replace('"', "'", $xmlFileContents));
    		$simpleXml = simplexml_load_string($xmlFileContents);
    		$json = json_encode($simpleXml);
    		return $json;
        } catch(\Exception $e) {
            throw $e;
        }
	}
	
	public function translateESocialJson($json) {
        try {
            $rows = array();
            $this->tagNivel = 0;
            $this->setTagsTraducao();
    		$this->translateESocialTag(json_decode($json,true), null, $rows);
    		return $rows;
        } catch(\Exception $e) {
            throw $e;
        }
	}
	
	public function translateESocialTag($tag, $parent, &$rows) {
        try {
            $this->tagNivel++;
	        $id = count($rows) + 1;
    		foreach ($tag as $name => $value) {
    		    if ($name === 'Signature') {
    		        // Não traduz a assinatura do evento
    		        continue;
    		    }
    		    elseif (is_array($value)) {
    		        $rows[] = array(
    		            'ID' => $id,
    		            'PARENT' => $parent,
    		            'NAME' => (is_int($name) ? $name + 1 : (isset($this->tagsTrad[$name]) ? $this->tagsTrad[$name] : $name)),
    		            'VALUE' => null,
    		            '_opened' => true,
    		            'NIVEL' => $this->tagNivel
		            );
    		        $this->translateESocialTag($value, $id, $rows);
    		        $id = count($rows) + 1;
    		    } else {
    		        $rows[] = array(
    		            'ID' => $id,
    		            'PARENT' => $parent,
    		            'NAME' => (isset($this->tagsTrad[$name]) ? $this->tagsTrad[$name] : $name),
    		            'VALUE' => array_key_exists($name, $this->tagsOpc) ? $value.' - '.$this->tagsOpc[$name][$value] : $value,
    		            '_opened' => true,
    		            'NIVEL' => $this->tagNivel
		            );
		            $id = $id + 1;
    		    }
    		}
        } catch(\Exception $e) {
            throw $e;
        }
	}
    
}