<?php
namespace HCM\Geral\Repository;

use Doctrine\DBAL\Connection;
use Doctrine\ORM\EntityManager;
use HCM\Util\Repositories;
use HCM\Util\DateUtil;
use HCM\Util\HCMParameters;

class ValidacaoSistemaRepository {

    public function __construct(Connection $connection, HCMParameters $hcmParameters, EntityManager $entityManager) {
        $this->connection = $connection;
        $this->hcmParameters = $hcmParameters;
        $this->entityManager = $entityManager;
    }
    
    public function retornaValidacaoSistema($params, $conditions){
        try{
            $where = '';
            if(isset($params['NRESTRUTLEGAL'])){
                $where .= ' AND VH.NRESTRUTLEGAL = :NRESTRUTLEGAL';
            }
            
            if(isset($params['NRESTRUTGEREN'])){
                $where .= ' AND VH.NRESTRUTGEREN = :NRESTRUTGEREN';
            }
            
            if(isset($params['NRTIPOVINCULOM'])){
                $where .= ' AND VM.NRTIPOVINCULOM = :NRTIPOVINCULOM';
            }
            
            if(isset($params['IDTIPOMSG'])){
                $where .= ' AND MSG.IDTIPOMSG = :IDTIPOMSG';
            }
            
            if(isset($params['NRVINCULOM'])){
                $where .= ' AND MSG.NRVINCULOM = :NRVINCULOM';
            }
            
            if(isset($params['DSORIGMSG'])){
                $where .= ' AND MSG.DSORIGMSG = :DSORIGMSG';
            }
            
            
            $sql = $this->entityManager->getConnection()->prepare("
                SELECT * FROM (
                    SELECT MSG.*, 
                           VM.NMVINCULOM,
                           ESTRUTLEGAL.NMESTRUTURAM AS NMESTRUTLEGAL,
                           ESTRUTGEREN.NMESTRUTURAM AS NMESTRUTGEREN                           

                    FROM FPA_MSGCALCULO MSG
                    
                    INNER JOIN GPE_VINCULOM VM
                        ON VM.NRVINCULOM   = MSG.NRVINCULOM
                        AND VM.NRORG       = MSG.NRORG
                    
                    INNER JOIN GPE_VINCULOH VH
                        ON VH.NRVINCULOM      = VM.NRVINCULOM
                        AND VH.NRORG          = VM.NRORG
                        AND VH.DTMESCOMPETENC =  (SELECT MAX(DTMESCOMPETENC)
                                                  FROM GPE_VINCULOH VHH
                                                  WHERE VHH.NRVINCULOM    = VH.NRVINCULOM
                                                  AND VHH.NRORG           = VH.NRORG
                                                  AND VHH.DTMESCOMPETENC <= :DTMESCOMPETENC
                                                  )

                    LEFT JOIN ESTRUTURAM ESTRUTLEGAL 
                        ON ESTRUTLEGAL.NRESTRUTURAM = VH.NRESTRUTLEGAL
                        AND ESTRUTLEGAL.NRORG = VH.NRORG
                    
                    LEFT JOIN ESTRUTURAM ESTRUTGEREN 
                        ON ESTRUTGEREN.NRESTRUTURAM = VH.NRESTRUTGEREN
                        AND ESTRUTGEREN.NRORG = VH.NRORG

                    WHERE MSG.NRORG      = :NRORG
                    AND MSG.NRCALCULOFOLHA = :NRCALCULOFOLHA
                    AND (VM.DTRESCISAOVINC is null OR VM.DTRESCISAOVINC >= :DTMESCOMPETENC)
                    AND (DSORIGMSG = 'ERROSCALCULO_VALIDA' OR DSORIGMSG = 'VALIDASISTEMA' OR DSORIGMSG = 'ERROESOPERIOD_VALIDA')
                    $where
                ) WHERE 1 = 1 $conditions
                ORDER BY NMVINCULOM"
            );
            $sql->execute($params);
            
            $items = $sql->fetchAll();
            
            return $items;
        } catch (\Exception $e) {            
            throw $e;
        }
    }
    
    public function retornaVinculos($dataSet){
        
        $params = array();
        $nrOrg = $this->hcmParameters->getNrorgTrabalho();
        $mesCompetencia = $this->hcmParameters->getDtCompetencia();
        
        $dtUltDiaMes = DateUtil::getUltimoDiaDoMes(DateUtil::getDataDeString($mesCompetencia));
        $params['p_dtultdiames'] = $dtUltDiaMes;
        $params['dtmescompetenc'] = $mesCompetencia; 
        
        $condIntervalo = null;
        $condProcesso = '(vm.dtrescisaovinc IS NULL
                                    OR vm.dtrescisaovinc >= :dtmescompetenc
                                  )
                                  AND ((vm.dtadmissaovinc <> vm.dtrescisaovinc) OR vm.dtrescisaovinc IS NULL)
                                  AND vm.dtadmissaovinc <=  :p_dtultdiames';
                                  
        if(count($dataSet['LIST_NRESTRUTLEGAL']) > 0){
            $params['p_nrestruturam'] = $dataSet["LIST_NRESTRUTLEGAL"];
            $condIntervalo = " vh.nrestrutlegal in (:p_nrestruturam)";
        }
        
        if(count($dataSet['LIST_NRESTRUTGEREN']) > 0){
            $params['p_nrestruturaGeren'] = $dataSet["LIST_NRESTRUTGEREN"];
            $condIntervalo .= $condIntervalo ? ' AND' : '';
            $condIntervalo .= " vh.nrestrutgeren in (:p_nrestruturaGeren)";
        }
        
        if(count($dataSet['NRVINCULOM_LIST']) > 0){
            $params['p_nrvinculom'] = $dataSet["NRVINCULOM_LIST"];
            $condIntervalo .= $condIntervalo ? ' AND' : '';
            $condIntervalo .= " vm.nrvinculom in (:p_nrvinculom)";
        }
        
        $vinculos = $this->entityManager->getRepository(Repositories::GPE_VINCULOM)->retornaVinculos($nrOrg, $mesCompetencia, $condProcesso, $condIntervalo, 'ORDER BY vh.nrestrutlegal, vh.nrestrutgeren', null, true, null, null, null, $params);
                
        return $vinculos;
    }
   
    public function getSomaValorEventosPorIncidencia($nrtipoincide, $nrorgPadrao, $nrorg, $nrVinculom, $nrtipomovimento, $nrTpModalidadeCalc, $mesCompetencia, $nrOcorrenciaCalc){
    
    $params = array('P_NRTIPOINCIDE'  => $nrtipoincide,
                    'P_NRORG_PADRAO'  => $nrorgPadrao,
                    'P_NRORG'         => $nrorg,
                    'P_NRVINCULOM'    => $nrVinculom,
                    'P_TIPOMOVIMENT'  => $nrtipomovimento,
                    'P_TPMODALIDCAL'  => $nrTpModalidadeCalc,
                    'P_DTMESCOMPETENC'=> $mesCompetencia,
                    'P_OCORRECAL'     => $nrOcorrenciaCalc);
    $query = "
            SELECT
            ROUND(SUM(NVL(DECODE(IEV.IDINCIDEVENTO, '+', ICF.VREVENTOFOLHA, '-', -ICF.VREVENTOFOLHA, 0), 0)), 2) VREVENTOFOLHA
            FROM FPA_CALCULOFOLHA  CF 
            JOIN FPA_INCIDEVENTO  IEV ON IEV.NRORG IN (CF.NRORG, :P_NRORG_PADRAO)
                AND IEV.NRTIPOINCIDE = :P_NRTIPOINCIDE
            JOIN FPA_ITECALCFOLHA ICF ON ICF.NRORG          = CF.NRORG
                AND ICF.NRVINCULOM     = :P_NRVINCULOM
                AND ICF.NREVENTOM      = IEV.NREVENTOM
                AND ICF.NRCALCULOFOLHA = CF.NRCALCULOFOLHA
            WHERE CF.NRORG          = :P_NRORG
                AND CF.NRTIPOMOVIMENT = :P_TIPOMOVIMENT
                AND CF.NRTPMODALIDCAL = :P_TPMODALIDCAL
                AND CF.DTOCORRENCIA   = :P_DTMESCOMPETENC
                AND CF.NROCORRECAL    = :P_OCORRECAL";
                
    return $this->connection->fetchAll($query, $params);
    
    }
    
    public function getEstruturaTomador($nrorg,$nrestruturam,$mescompetencia){
        
        $params = array('P_NRORG'         => $nrorg,
                        'P_NRESTRUTURAM'    => $nrestruturam);
        $query = "
            SELECT ESUP.NRESTRUTURAM
            FROM HIERARQUIAORG H 
            LEFT JOIN ESTRUTURAM  EREL ON H.NRESTRUTURAMREL = EREL.NRESTRUTURAM AND EREL.NRORG = H.NRORG 
            LEFT JOIN ESTRUTURAM  ESUP ON H.NRESTRUTURAMSUP = ESUP.NRESTRUTURAM AND ESUP.NRORG = H.NRORG 
            LEFT JOIN TIPOHIERARQORG TPH ON H.NRTIPOHIERARQORG = TPH.NRTIPOHIERARQORG AND H.NRORG = TPH.NRORG
            WHERE H.NRORG = :P_NRORG
            AND EREL.NRESTRUTURAM = :P_NRESTRUTURAM
            AND TPH.NRTIPOESTRUTSUP = 7
            AND TPH.NRTIPOESTRUTREL = EREL.NRTIPOESTRUTURA
        ";
                    
        $return = $this->connection->fetchAll($query, $params);
        if(empty($return))
            return false;
        
        $estruturaTom = $this->entityManager->getRepository(Repositories::ESTRUTURAM)->retornaEstrutura($nrorg,  $return[0]['NRESTRUTURAM'], $mescompetencia);
        if(!$estruturaTom)
            return false;
        $historicoTom = $this->entityManager->getRepository(Repositories::ESTRUTURAH)->retornaEstruturaHistorico($nrorg, $return[0]['NRESTRUTURAM'], $mescompetencia);
        $estruturaTom->historico = $historicoTom;
        return $estruturaTom;
        
        
    }
    
    public function getGrupoIncidencia($nrorg, $nrestruturalegal, $dtmescompetencia){
        
    $params = array('P_NRORG'         => $nrorg,
                    'P_NRESTRUTLEGAL' => $nrestruturalegal,
                    'P_DTMESCOMPETENC'=> $dtmescompetencia);
    
    $query = '  SELECT 
                    NRTIPOINCIDE, NREVENTOM, VRPERCENT, VRMULTIPLIC, VRDIVIDE
                FROM GPE_PARAMESTRUTM PEM
                    LEFT JOIN GPE_PARAMESTRUTH PEH
                        ON PEH.NRORG = PEM.NRORG
                        AND PEH.NRPARAMESTRUTM = PEM.NRPARAMESTRUTM
                        AND PEH.DTMESCOMPETENC = (SELECT MAX(DTMESCOMPETENC)
                                                  FROM GPE_PARAMESTRUTH
                                                  WHERE DTMESCOMPETENC <= :P_DTMESCOMPETENC
                                                  AND NRORG = PEM.NRORG
                                                  AND NRPARAMESTRUTM = PEH.NRPARAMESTRUTM ) 
                    LEFT JOIN FPA_PARAMFOLHAH PFH
                        ON PFH.NRORG = PEM.NRORG
                        AND PFH.NRPARAMFOLHAM = PEH.NRPARAMFOLHAM
                        AND PFH.DTMESCOMPETENC = (SELECT MAX(DTMESCOMPETENC)
                                                  FROM FPA_PARAMFOLHAH
                                                  WHERE DTMESCOMPETENC <= :P_DTMESCOMPETENC
                                                  AND NRORG = PEM.NRORG
                                                  AND NRPARAMFOLHAM = PFH.NRPARAMFOLHAM )
                    LEFT JOIN FPA_GRPITEM GIT
                        ON ( GIT.NRORG = PEM.NRORG
                        OR GIT.NRORG = 0 )
                        AND GIT.NRGRPINCIDE = PFH.NRGRPINCIDE
                WHERE PEM.NRORG = :P_NRORG
                    AND PEM.NRESTRUTURAM = :P_NRESTRUTLEGAL
                    AND PEM.DTINIVIGENCIA = (SELECT MAX(DTINIVIGENCIA)
                                             FROM GPE_PARAMESTRUTM
                                             WHERE DTINIVIGENCIA <= :P_DTMESCOMPETENC
                                             AND NRORG = PEM.NRORG
                                             AND NRESTRUTURAM = PEM.NRESTRUTURAM )';    
    return $this->connection->fetchAll($query, $params);
        
    }
    
    public function getDiasTrabalhados($mesCompetencia, $nrorg, $nrVinculom, $nrEventom, $nrOcorrenciaCalc, $nrtipomovimento, $nrTpModalidadeCalc){
        
        $params = array('P_DTMESCOMPETENC' => $mesCompetencia,
                        'P_NRORG'          => $nrorg,
                        'P_NRVINCULOM'     => $nrVinculom,
                        ':P_OCORRECAL'     => $nrOcorrenciaCalc,
                        'P_TIPOMOVIMENT'   => $nrtipomovimento,
                        'P_TPMODALIDCAL'   => $nrTpModalidadeCalc);
        
        
        $query = "
        SELECT
            DIAS_SALARIO_ITEM,
            DECODE(DIAS_QUERY_TRABALHADOS, DIAS_COMP, 30, DIAS_QUERY_TRABALHADOS) DIAS_QUERY_TRABALHADOS
        FROM (SELECT
                LAST_DAY(TO_DATE(:P_DTMESCOMPETENC, 'DD/MM/YYYY')) - TO_DATE(:P_DTMESCOMPETENC, 'DD/MM/YYYY') + 1 DIAS_COMP,
                RES.DIAS_SALARIO_ITEM,
                RES.FIM - RES.INI + 1 - NVL(RES.FIM_FER - RES.INI_FER + 1, 0) DIAS_QUERY_TRABALHADOS
             FROM (SELECT
                    VM.NRVINCULOM,
                    VM.DTADMISSAOVINC,
                    VM.DTRESCISAOVINC,
                   CASE
                    WHEN VM.DTADMISSAOVINC > TO_DATE(:P_DTMESCOMPETENC, 'DD/MM/YYYY') THEN TRUNC(VM.DTADMISSAOVINC, 'DD')
                    ELSE TO_DATE(:P_DTMESCOMPETENC, 'DD/MM/YYYY')
                   END INI,
                   CASE
                    WHEN VM.DTRESCISAOVINC < LAST_DAY(TO_DATE(:P_DTMESCOMPETENC, 'DD/MM/YYYY')) THEN TRUNC(VM.DTRESCISAOVINC, 'DD')
                    ELSE LAST_DAY(TO_DATE(:P_DTMESCOMPETENC, 'DD/MM/YYYY'))
                   END FIM,
                  (SELECT VRREFFOLHA FROM FPA_ITECALCFOLHA
                    WHERE NRORG          = CF.NRORG
                      AND NRVINCULOM     = VM.NRVINCULOM
                      AND NREVENTOM      IN (".$nrEventom.")
                      AND NRCALCULOFOLHA = CF.NRCALCULOFOLHA) DIAS_SALARIO_ITEM,
                  CASE
                    WHEN FER.DTINIGOZOFERIAS IS NULL                                    THEN NULL
                    WHEN FER.DTINIGOZOFERIAS > TO_DATE(:P_DTMESCOMPETENC, 'DD/MM/YYYY') THEN TRUNC(FER.DTINIGOZOFERIAS, 'DD')
                    ELSE TO_DATE(:P_DTMESCOMPETENC, 'DD/MM/YYYY')
                  END INI_FER,
                  CASE
                    WHEN FER.DTFIMGOZOFERIAS IS NULL                                    THEN NULL
                    WHEN FER.DTFIMGOZOFERIAS < LAST_DAY(TO_DATE(:P_DTMESCOMPETENC, 'DD/MM/YYYY')) THEN TRUNC(FER.DTFIMGOZOFERIAS, 'DD')
                    ELSE LAST_DAY(TO_DATE(:P_DTMESCOMPETENC, 'DD/MM/YYYY'))
                  END FIM_FER
                FROM FPA_CALCULOFOLHA  CF 
                JOIN GPE_VINCULOM VM ON VM.NRORG      = CF.NRORG
                                    AND VM.NRVINCULOM = :P_NRVINCULOM      
                /*Férias*/
                LEFT JOIN ( SELECT GFE.DTINIGOZOFERIAS, GFE.DTFIMGOZOFERIAS
                            FROM FPA_FERIAS FE
                            JOIN FPA_GOZOFERIAS GFE ON GFE.NRORG           = FE.NRORG
                                                   AND GFE.NRFERIAS        = FE.NRFERIAS
                                                   AND ( GFE.DTINIGOZOFERIAS BETWEEN TO_DATE(:P_DTMESCOMPETENC, 'DD/MM/YYYY') AND LAST_DAY(TO_DATE(:P_DTMESCOMPETENC, 'DD/MM/YYYY'))
                                                      OR GFE.DTFIMGOZOFERIAS BETWEEN TO_DATE(:P_DTMESCOMPETENC, 'DD/MM/YYYY') AND LAST_DAY(TO_DATE(:P_DTMESCOMPETENC, 'DD/MM/YYYY')) 
                                                      OR ( TO_CHAR(GFE.DTINIGOZOFERIAS, 'DD/MM/YYYY') = '31/01' || SUBSTR(:P_DTMESCOMPETENC, 6, 5) AND SUBSTR(:P_DTMESCOMPETENC, 0, 5) = '01/02' ))
                            WHERE FE.NRORG      = :P_NRORG
                              AND FE.NRVINCULOM = :P_NRVINCULOM ) FER ON 1 = 1
                                       
                WHERE CF.NRORG          = :P_NRORG
                  AND CF.NRTIPOMOVIMENT = :P_TIPOMOVIMENT
                  AND CF.NRTPMODALIDCAL = :P_TPMODALIDCAL
                  AND CF.DTOCORRENCIA   = :P_DTMESCOMPETENC
                  AND CF.NROCORRECAL    = :P_OCORRECAL
        ) RES
    )
";
    return $this->connection->fetchAll($query, $params);
        
    }
    
    public function getPeriodoFerias($mesCompetencia, $nrorg, $nrVinculom){
        
        $params = array('P_DTMESCOMPETENC' => $mesCompetencia,
                        'P_NRORG'          => $nrorg,
                        'P_NRVINCULOM'     => $nrVinculom);
        
$query = <<<SQL
SELECT 
    TRUNC(GFE.DTINIGOZOFERIAS, 'MM')                 COMPETENCIA1, 
    ADD_MONTHS(TRUNC(GFE.DTINIGOZOFERIAS, 'MM'), -1) COMPETENCIA2,
    ADD_MONTHS(TRUNC(GFE.DTFIMGOZOFERIAS, 'MM'), -1) COMPETENCIA3,
    TRUNC(GFE.DTFIMGOZOFERIAS, 'MM')                 COMPETENCIA4, 
    GFE.DTINIGOZOFERIAS,
    GFE.DTFIMGOZOFERIAS
FROM FPA_FERIAS FE
JOIN FPA_GOZOFERIAS GFE ON GFE.NRORG           = FE.NRORG
                       AND GFE.NRFERIAS        = FE.NRFERIAS
                       AND ( TRUNC(GFE.DTINIGOZOFERIAS, 'DD') BETWEEN TO_DATE(:P_DTMESCOMPETENC, 'DD/MM/YYYY') AND LAST_DAY(TO_DATE(:P_DTMESCOMPETENC, 'DD/MM/YYYY'))
                          OR TRUNC(GFE.DTFIMGOZOFERIAS, 'DD') BETWEEN TO_DATE(:P_DTMESCOMPETENC, 'DD/MM/YYYY') AND LAST_DAY(TO_DATE(:P_DTMESCOMPETENC, 'DD/MM/YYYY'))
                          OR ( TO_CHAR(GFE.DTINIGOZOFERIAS, 'DD/MM/YYYY') = '31/01' || SUBSTR(:P_DTMESCOMPETENC, 6, 5) AND SUBSTR(:P_DTMESCOMPETENC, 0, 5) = '01/02' ) )
WHERE FE.NRORG      = :P_NRORG
  AND FE.NRVINCULOM = :P_NRVINCULOM
SQL;
        
    return $this->connection->fetchAll($query, $params);
        
    }
    public function getCalculoIRRF($nrorg, $baseInss, $valorInss, $mesCompetencia, $nrVinculom, $nrtabelafolha){
        
        $params = array('P_NRTABELAFOLHA'  => $nrtabelafolha,
                        'P_DTMESCOMPETENC' => $mesCompetencia,
                        'P_NRVINCULOM'     => $nrVinculom,
                        'P_NRORG'          => $nrorg);
        $query = "
SELECT (SELECT (VALOR * (FT.VRPECALCULORESU/100)) + FT.VRAPLICARESULT
        FROM FPA_FAIXATABELA FT
        WHERE FT.NRTABELAFOLHA = :P_NRTABELAFOLHA
          AND FT.DTINIVIGENCIA = ( SELECT MAX(DTINIVIGENCIA) FROM FPA_FAIXATABELA
                                   WHERE NRTABELAFOLHA = FT.NRTABELAFOLHA
                                     AND DTINIVIGENCIA <= :P_DTMESCOMPETENC )
          AND FT.VRLIMITSUPFAIXA = ( SELECT MIN(VRLIMITSUPFAIXA) FROM FPA_FAIXATABELA
                                     WHERE NRTABELAFOLHA = FT.NRTABELAFOLHA
                                       AND DTINIVIGENCIA = FT.DTINIVIGENCIA
                                       AND VRLIMITSUPFAIXA >= VALOR) ) VALOR
FROM (  SELECT (".$baseInss." - ".$valorInss.") - (VH.NRDEPENDIR * VRH.VRREFERENCIA) VALOR
        FROM GPE_VINCULOM VM 
        JOIN GPE_VINCULOH VH ON VH.NRORG      = VM.NRORG
                        AND VH.NRVINCULOM = VM.NRVINCULOM
                        AND VH.DTMESCOMPETENC = ( SELECT MAX(DTMESCOMPETENC) FROM GPE_VINCULOH
                                                  WHERE DTMESCOMPETENC <= :P_DTMESCOMPETENC
                                                    AND NRORG      = VH.NRORG
                                                    AND NRVINCULOM = VH.NRVINCULOM )
        LEFT JOIN FPA_VALORREFERENCIAM VRM ON ( VRM.NRORG = 0 OR VRM.NRORG = VM.NRORG )
                                  AND VRM.DSREFERENCIA = 'VR_DEDUCAO_DEP_IR'
                                  AND VRM.DTINIVIGENCIA = ( SELECT MAX(DTINIVIGENCIA) FROM FPA_VALORREFERENCIAM
                                                            WHERE ( NRORG = 0 OR NRORG = VRM.NRORG )
                                                              AND DSREFERENCIA   = VRM.DSREFERENCIA
                                                              AND DTINIVIGENCIA <= :P_DTMESCOMPETENC )
        LEFT JOIN FPA_VALORREFERENCIAH VRH ON VRH.NRVALORREFERENCIAM = VRM.NRVALORREFERENCIAM
                                  AND VRH.DTMESCOMPETENC = ( SELECT MAX(DTMESCOMPETENC) FROM FPA_VALORREFERENCIAH
                                                             WHERE NRVALORREFERENCIAM = VRM.NRVALORREFERENCIAM
                                                               AND DTMESCOMPETENC    <= :P_DTMESCOMPETENC )
        WHERE VM.NRORG = :P_NRORG
        AND VM.NRVINCULOM = :P_NRVINCULOM
)";
        return $this->connection->fetchAll($query, $params);
    }
    
    
    public function getBaseIRRF($nrorg, $baseInss, $valorInss, $mesCompetencia, $nrVinculom){
        
        $params = array('P_DTMESCOMPETENC' => $mesCompetencia,
                        'P_NRVINCULOM'     => $nrVinculom,
                        'P_NRORG'          => $nrorg);
     
        $query = "SELECT (".$baseInss." - ".$valorInss.") - (VH.NRDEPENDIR * VRH.VRREFERENCIA) VALOR
        FROM GPE_VINCULOM VM 
        JOIN GPE_VINCULOH VH ON VH.NRORG      = VM.NRORG
                        AND VH.NRVINCULOM = VM.NRVINCULOM
                        AND VH.DTMESCOMPETENC = ( SELECT MAX(DTMESCOMPETENC) FROM GPE_VINCULOH
                                                  WHERE DTMESCOMPETENC <= :P_DTMESCOMPETENC
                                                    AND NRORG      = VH.NRORG
                                                    AND NRVINCULOM = VH.NRVINCULOM )
        LEFT JOIN FPA_VALORREFERENCIAM VRM ON ( VRM.NRORG = 0 OR VRM.NRORG = VM.NRORG )
                                  AND VRM.DSREFERENCIA = 'VR_DEDUCAO_DEP_IR'
                                  AND VRM.DTINIVIGENCIA = ( SELECT MAX(DTINIVIGENCIA) FROM FPA_VALORREFERENCIAM
                                                            WHERE ( NRORG = 0 OR NRORG = VRM.NRORG )
                                                              AND DSREFERENCIA   = VRM.DSREFERENCIA
                                                              AND DTINIVIGENCIA <= :P_DTMESCOMPETENC )
        LEFT JOIN FPA_VALORREFERENCIAH VRH ON VRH.NRVALORREFERENCIAM = VRM.NRVALORREFERENCIAM
                                  AND VRH.DTMESCOMPETENC = ( SELECT MAX(DTMESCOMPETENC) FROM FPA_VALORREFERENCIAH
                                                             WHERE NRVALORREFERENCIAM = VRM.NRVALORREFERENCIAM
                                                               AND DTMESCOMPETENC    <= :P_DTMESCOMPETENC )
        WHERE VM.NRORG = :P_NRORG
        AND VM.NRVINCULOM = :P_NRVINCULOM ";
        
        return $this->connection->fetchAll($query, $params);
        
    }
    
    public function getBaseCalculoEncargoEsocial($parametros){
        
        if($parametros['TPINCIDENCIAESO'] == 'INSS'){
            $incideciaEso = 'JOIN INCIDENCIACP INCIDENCIAESO ON INCIDENCIAESO.CDESOCIAL = :P_CDINCIDESOCIAL';
            $parametros['NRGRPINCIDE'] = '46';
        }else if($parametros['TPINCIDENCIAESO'] == 'IRRF'){
            $incideciaEso = 'JOIN INCIDENCIAIRRF INCIDENCIAESO ON INCIDENCIAESO.CDESOCIAL = :P_CDINCIDESOCIAL';
            $parametros['NRGRPINCIDE'] = '47';
        }else if($parametros['TPINCIDENCIAESO'] == 'FGTS'){
            $incideciaEso = 'JOIN INCIDENCIAFGTS INCIDENCIAESO ON INCIDENCIAESO.CDESOCIAL = :P_CDINCIDESOCIAL';
            $parametros['NRGRPINCIDE'] = '48';
        }
        
        $arrayEvent = explode(',', $parametros['NREVENTOMBASE']);
        $arrayincid = explode(',', $parametros['INCIDENCIABASE']);
        $itensCalculo = "JOIN FPA_ITECALCFOLHA BASE ON BASE.NRCALCULOFOLHA = :P_NRCALCULOFOLHA
                                                       AND BASE.NREVENTOM in (".$parametros['NREVENTOMBASE'].")
                                                       AND ((BASE.NREVENTOM = ".$arrayEvent[0]." AND FPA_TIPOINCIDE.NRTIPOINCIDE = ".$arrayincid[0].")
                                                           OR (BASE.NREVENTOM = ".$arrayEvent[1]." AND FPA_TIPOINCIDE.NRTIPOINCIDE = ".$arrayincid[1]."))
                                                       AND BASE.NRVINCULOM     = GPE_VINCULOM.NRVINCULOM";
        
        $sql = <<<SQL
SELECT DISTINCT FPA_ITECALCFOLHA.NRVINCULOM,
                SUM(FPA_ITECALCFOLHA.VREVENTOFOLHA * DECODE(FPA_INCIDEVENTO.IDINCIDEVENTO, '+', 1, '-', -1, 0)) VRSOMA,
                BASE.VREVENTOFOLHA VRBASE,
                BASE.NREVENTOM NREVENTOBASE
           FROM GPE_VINCULOM 
           $incideciaEso
           JOIN FPA_TIPOINCIDE ON INSTR(','||INCIDENCIAESO.DSNRTIPOINCIDE||',',','||FPA_TIPOINCIDE.NRTIPOINCIDE||',') > 0
                              AND FPA_TIPOINCIDE.NRORG IN (GPE_VINCULOM.NRORG, 0)
           JOIN FPA_INCIDEVENTO ON FPA_INCIDEVENTO.NRTIPOINCIDE = FPA_TIPOINCIDE.NRTIPOINCIDE
           JOIN FPA_ITECALCFOLHA ON FPA_ITECALCFOLHA.NRCALCULOFOLHA = :P_NRCALCULOFOLHA
                                AND FPA_ITECALCFOLHA.NREVENTOM = FPA_INCIDEVENTO.NREVENTOM
                                AND FPA_ITECALCFOLHA.NRVINCULOM = GPE_VINCULOM.NRVINCULOM
           $itensCalculo
          WHERE GPE_VINCULOM.NRORG = :P_NRORG
            AND GPE_VINCULOM.NRVINCULOM = :P_NRVINCULOM
            AND (FPA_INCIDEVENTO.NRTIPOINCIDE IN (SELECT NRTIPOINCIDE 
                                                    FROM FPA_GRPITEM 
                                                   WHERE NRGRPINCIDE = :P_NRGRPINCIDE)
                 OR FPA_TIPOINCIDE.DSNRTIPOMOVIMENT IS NOT NULL )
       GROUP BY FPA_ITECALCFOLHA.NRVINCULOM, BASE.VREVENTOFOLHA, BASE.NREVENTOM, FPA_TIPOINCIDE.NRTIPOINCIDE
SQL;
        $params = array('P_NRCALCULOFOLHA' => $parametros['NRCALCULOFOLHA'],
                        'P_NRVINCULOM'     => $parametros['NRVINCULOM'],
                        'P_NRORG'          => $parametros['NRORG'],
                        'P_CDINCIDESOCIAL' => $parametros['CDINCIDESOCIAL'],
                        'P_NRGRPINCIDE'    => $parametros['NRGRPINCIDE']);
        return $this->connection->fetchAssoc($sql, $params);

    }
} 